#ifndef __SOURCE4_AUTH_NTLM_AUTH_PROTO_H__
#define __SOURCE4_AUTH_NTLM_AUTH_PROTO_H__

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
/* This file was automatically generated by mkproto.pl. DO NOT EDIT */

/* this file contains prototypes for functions that are private 
 * to this subsystem or library. These functions should not be 
 * used outside this particular subsystem! */


/* The following definitions come from ../../source4/auth/ntlm/auth.c  */

const char **auth_methods_from_lp(TALLOC_CTX *mem_ctx, struct loadparm_context *lp_ctx);
const struct auth_operations *auth_backend_byname(const char *name);
const struct auth_critical_sizes *auth_interface_version(void);

/* The following definitions come from ../../source4/auth/ntlm/auth_util.c  */


/****************************************************************************
 Create an auth_usersupplied_data structure after appropriate mapping.
****************************************************************************/
NTSTATUS encrypt_user_info(TALLOC_CTX *mem_ctx, struct auth4_context *auth_context, 
			   enum auth_password_state to_state,
			   const struct auth_usersupplied_info *user_info_in,
			   const struct auth_usersupplied_info **user_info_encrypted);

/* The following definitions come from ../../source4/auth/ntlm/auth_simple.c  */

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2)

#endif /* __SOURCE4_AUTH_NTLM_AUTH_PROTO_H__ */

