/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/eventlog6.h"

#ifndef _HEADER_NDR_eventlog6
#define _HEADER_NDR_eventlog6

#define NDR_EVENTLOG6_UUID "f6beaff7-1e19-4fbb-9f8f-b89e2018337c"
#define NDR_EVENTLOG6_VERSION 1.0
#define NDR_EVENTLOG6_NAME "eventlog6"
#define NDR_EVENTLOG6_HELPSTRING "Eventlog6"
extern const struct ndr_interface_table ndr_table_eventlog6;
#define NDR_EVENTLOG6_EVTRPCREGISTERREMOTESUBSCRIPTION (0x00)

#define NDR_EVENTLOG6_EVTRPCREMOTESUBSCRIPTIONNEXTASYNC (0x01)

#define NDR_EVENTLOG6_EVTRPCREMOTESUBSCRIPTIONNEXT (0x02)

#define NDR_EVENTLOG6_EVTRPCREMOTESUBSCRIPTIONWAITASYNC (0x03)

#define NDR_EVENTLOG6_EVTRPCREGISTERCONTROLLABLEOPERATION (0x04)

#define NDR_EVENTLOG6_EVTRPCREGISTERLOGQUERY (0x05)

#define NDR_EVENTLOG6_EVTRPCCLEARLOG (0x06)

#define NDR_EVENTLOG6_EVTRPCEXPORTLOG (0x07)

#define NDR_EVENTLOG6_EVTRPCLOCALIZEEXPORTLOG (0x08)

#define NDR_EVENTLOG6_EVTRPCMESSAGERENDER (0x09)

#define NDR_EVENTLOG6_EVTRPCMESSAGERENDERDEFAULT (0x0a)

#define NDR_EVENTLOG6_EVTRPCQUERYNEXT (0x0b)

#define NDR_EVENTLOG6_EVTRPCQUERYSEEK (0x0c)

#define NDR_EVENTLOG6_EVTRPCCLOSE (0x0d)

#define NDR_EVENTLOG6_EVTRPCCANCEL (0x0e)

#define NDR_EVENTLOG6_EVTRPCASSERTCONFIG (0x0f)

#define NDR_EVENTLOG6_EVTRPCRETRACTCONFIG (0x10)

#define NDR_EVENTLOG6_EVTRPCOPENLOGHANDLE (0x11)

#define NDR_EVENTLOG6_EVTRPCGETLOGFILEINFO (0x12)

#define NDR_EVENTLOG6_EVTRPCGETCHANNELLIST (0x13)

#define NDR_EVENTLOG6_EVTRPCGETCHANNELCONFIG (0x14)

#define NDR_EVENTLOG6_EVTRPCPUTCHANNELCONFIG (0x15)

#define NDR_EVENTLOG6_EVTRPCGETPUBLISHERLIST (0x16)

#define NDR_EVENTLOG6_EVTRPCGETPUBLISHERLISTFORCHANNEL (0x17)

#define NDR_EVENTLOG6_EVTRPCGETPUBLISHERMETADATA (0x18)

#define NDR_EVENTLOG6_EVTRPCGETPUBLISHERRESOURCEMETADATA (0x19)

#define NDR_EVENTLOG6_EVTRPCGETEVENTMETADATAENUM (0x1a)

#define NDR_EVENTLOG6_EVTRPCGETNEXTEVENTMETADATA (0x1b)

#define NDR_EVENTLOG6_EVTRPCGETCLASSICLOGDISPLAYNAME (0x1c)

#define NDR_EVENTLOG6_CALL_COUNT (29)
void ndr_print_eventlog6_RpcInfo(struct ndr_print *ndr, const char *name, const struct eventlog6_RpcInfo *r);
void ndr_print_eventlog6_boolean8Array(struct ndr_print *ndr, const char *name, const struct eventlog6_boolean8Array *r);
void ndr_print_eventlog6_UInt32Array(struct ndr_print *ndr, const char *name, const struct eventlog6_UInt32Array *r);
void ndr_print_eventlog6_UInt64Array(struct ndr_print *ndr, const char *name, const struct eventlog6_UInt64Array *r);
void ndr_print_eventlog6_StringArray(struct ndr_print *ndr, const char *name, const struct eventlog6_StringArray *r);
void ndr_print_eventlog6_GuidArray(struct ndr_print *ndr, const char *name, const struct eventlog6_GuidArray *r);
void ndr_print_eventlog6_EvtRpcVariantType(struct ndr_print *ndr, const char *name, enum eventlog6_EvtRpcVariantType r);
void ndr_print_eventlog6_EvtRpcVariantUnion(struct ndr_print *ndr, const char *name, const union eventlog6_EvtRpcVariantUnion *r);
void ndr_print_eventlog6_EvtRpcVariant(struct ndr_print *ndr, const char *name, const struct eventlog6_EvtRpcVariant *r);
void ndr_print_eventlog6_EvtRpcVariantList(struct ndr_print *ndr, const char *name, const struct eventlog6_EvtRpcVariantList *r);
void ndr_print_eventlog6_EvtRpcQueryChannelInfo(struct ndr_print *ndr, const char *name, const struct eventlog6_EvtRpcQueryChannelInfo *r);
void ndr_print_eventlog6_EvtRpcRegisterRemoteSubscription(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcRegisterRemoteSubscription *r);
void ndr_print_eventlog6_EvtRpcRemoteSubscriptionNextAsync(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcRemoteSubscriptionNextAsync *r);
void ndr_print_eventlog6_EvtRpcRemoteSubscriptionNext(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcRemoteSubscriptionNext *r);
void ndr_print_eventlog6_EvtRpcRemoteSubscriptionWaitAsync(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcRemoteSubscriptionWaitAsync *r);
void ndr_print_eventlog6_EvtRpcRegisterControllableOperation(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcRegisterControllableOperation *r);
void ndr_print_eventlog6_EvtRpcRegisterLogQuery(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcRegisterLogQuery *r);
void ndr_print_eventlog6_EvtRpcClearLog(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcClearLog *r);
void ndr_print_eventlog6_EvtRpcExportLog(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcExportLog *r);
void ndr_print_eventlog6_EvtRpcLocalizeExportLog(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcLocalizeExportLog *r);
void ndr_print_eventlog6_EvtRpcMessageRender(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcMessageRender *r);
void ndr_print_eventlog6_EvtRpcMessageRenderDefault(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcMessageRenderDefault *r);
void ndr_print_eventlog6_EvtRpcQueryNext(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcQueryNext *r);
void ndr_print_eventlog6_EvtRpcQuerySeek(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcQuerySeek *r);
void ndr_print_eventlog6_EvtRpcClose(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcClose *r);
void ndr_print_eventlog6_EvtRpcCancel(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcCancel *r);
void ndr_print_eventlog6_EvtRpcAssertConfig(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcAssertConfig *r);
void ndr_print_eventlog6_EvtRpcRetractConfig(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcRetractConfig *r);
void ndr_print_eventlog6_EvtRpcOpenLogHandle(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcOpenLogHandle *r);
void ndr_print_eventlog6_EvtRpcGetLogFileInfo(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcGetLogFileInfo *r);
void ndr_print_eventlog6_EvtRpcGetChannelList(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcGetChannelList *r);
void ndr_print_eventlog6_EvtRpcGetChannelConfig(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcGetChannelConfig *r);
void ndr_print_eventlog6_EvtRpcPutChannelConfig(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcPutChannelConfig *r);
void ndr_print_eventlog6_EvtRpcGetPublisherList(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcGetPublisherList *r);
void ndr_print_eventlog6_EvtRpcGetPublisherListForChannel(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcGetPublisherListForChannel *r);
void ndr_print_eventlog6_EvtRpcGetPublisherMetadata(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcGetPublisherMetadata *r);
void ndr_print_eventlog6_EvtRpcGetPublisherResourceMetadata(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcGetPublisherResourceMetadata *r);
void ndr_print_eventlog6_EvtRpcGetEventMetadataEnum(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcGetEventMetadataEnum *r);
void ndr_print_eventlog6_EvtRpcGetNextEventMetadata(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcGetNextEventMetadata *r);
void ndr_print_eventlog6_EvtRpcGetClassicLogDisplayName(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct eventlog6_EvtRpcGetClassicLogDisplayName *r);
#endif /* _HEADER_NDR_eventlog6 */
