/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/initshutdown.h"

#ifndef _HEADER_NDR_initshutdown
#define _HEADER_NDR_initshutdown

#define NDR_INITSHUTDOWN_UUID "894de0c0-0d55-11d3-a322-00c04fa321a1"
#define NDR_INITSHUTDOWN_VERSION 1.0
#define NDR_INITSHUTDOWN_NAME "initshutdown"
#define NDR_INITSHUTDOWN_HELPSTRING "Init shutdown service"
extern const struct ndr_interface_table ndr_table_initshutdown;
#define NDR_INITSHUTDOWN_INIT (0x00)

#define NDR_INITSHUTDOWN_ABORT (0x01)

#define NDR_INITSHUTDOWN_INITEX (0x02)

#define NDR_INITSHUTDOWN_CALL_COUNT (3)
void ndr_print_initshutdown_Init(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct initshutdown_Init *r);
void ndr_print_initshutdown_Abort(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct initshutdown_Abort *r);
void ndr_print_initshutdown_InitEx(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct initshutdown_InitEx *r);
#endif /* _HEADER_NDR_initshutdown */
