/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/fsrvp.h"

#ifndef _HEADER_NDR_FileServerVssAgent
#define _HEADER_NDR_FileServerVssAgent

#define NDR_FILESERVERVSSAGENT_UUID "a8e0653c-2744-4389-a61d-7373df8b2292"
#define NDR_FILESERVERVSSAGENT_VERSION 1.0
#define NDR_FILESERVERVSSAGENT_NAME "FileServerVssAgent"
#define NDR_FILESERVERVSSAGENT_HELPSTRING "File Server Remote VSS Protocol"
extern const struct ndr_interface_table ndr_table_FileServerVssAgent;
#define NDR_FSS_GETSUPPORTEDVERSION (0x00)

#define NDR_FSS_SETCONTEXT (0x01)

#define NDR_FSS_STARTSHADOWCOPYSET (0x02)

#define NDR_FSS_ADDTOSHADOWCOPYSET (0x03)

#define NDR_FSS_COMMITSHADOWCOPYSET (0x04)

#define NDR_FSS_EXPOSESHADOWCOPYSET (0x05)

#define NDR_FSS_RECOVERYCOMPLETESHADOWCOPYSET (0x06)

#define NDR_FSS_ABORTSHADOWCOPYSET (0x07)

#define NDR_FSS_ISPATHSUPPORTED (0x08)

#define NDR_FSS_ISPATHSHADOWCOPIED (0x09)

#define NDR_FSS_GETSHAREMAPPING (0x0a)

#define NDR_FSS_DELETESHAREMAPPING (0x0b)

#define NDR_FSS_PREPARESHADOWCOPYSET (0x0c)

#define NDR_FILESERVERVSSAGENT_CALL_COUNT (13)
void ndr_print_fssagent_share_mapping_1(struct ndr_print *ndr, const char *name, const struct fssagent_share_mapping_1 *r);
void ndr_print_fssagent_share_mapping(struct ndr_print *ndr, const char *name, const union fssagent_share_mapping *r);
void ndr_print_fss_GetSupportedVersion(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct fss_GetSupportedVersion *r);
void ndr_print_fss_SetContext(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct fss_SetContext *r);
void ndr_print_fss_StartShadowCopySet(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct fss_StartShadowCopySet *r);
void ndr_print_fss_AddToShadowCopySet(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct fss_AddToShadowCopySet *r);
void ndr_print_fss_CommitShadowCopySet(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct fss_CommitShadowCopySet *r);
void ndr_print_fss_ExposeShadowCopySet(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct fss_ExposeShadowCopySet *r);
void ndr_print_fss_RecoveryCompleteShadowCopySet(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct fss_RecoveryCompleteShadowCopySet *r);
void ndr_print_fss_AbortShadowCopySet(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct fss_AbortShadowCopySet *r);
void ndr_print_fss_IsPathSupported(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct fss_IsPathSupported *r);
void ndr_print_fss_IsPathShadowCopied(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct fss_IsPathShadowCopied *r);
void ndr_print_fss_GetShareMapping(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct fss_GetShareMapping *r);
void ndr_print_fss_DeleteShareMapping(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct fss_DeleteShareMapping *r);
void ndr_print_fss_PrepareShadowCopySet(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct fss_PrepareShadowCopySet *r);
#endif /* _HEADER_NDR_FileServerVssAgent */
