#ifndef __NDR_WINREG_SCOMPAT_H__
#define __NDR_WINREG_SCOMPAT_H__

struct pipes_struct;
struct dcesrv_endpoint_server;
struct dcesrv_call_state;

const struct dcesrv_endpoint_server *winreg_get_ep_server(void);

NTSTATUS winreg__op_ndr_pull(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, struct ndr_pull *pull, void **r);
struct winreg_OpenHKCR;
WERROR _winreg_OpenHKCR(struct pipes_struct *p, struct winreg_OpenHKCR *r);
struct winreg_OpenHKCU;
WERROR _winreg_OpenHKCU(struct pipes_struct *p, struct winreg_OpenHKCU *r);
struct winreg_OpenHKLM;
WERROR _winreg_OpenHKLM(struct pipes_struct *p, struct winreg_OpenHKLM *r);
struct winreg_OpenHKPD;
WERROR _winreg_OpenHKPD(struct pipes_struct *p, struct winreg_OpenHKPD *r);
struct winreg_OpenHKU;
WERROR _winreg_OpenHKU(struct pipes_struct *p, struct winreg_OpenHKU *r);
struct winreg_CloseKey;
WERROR _winreg_CloseKey(struct pipes_struct *p, struct winreg_CloseKey *r);
struct winreg_CreateKey;
WERROR _winreg_CreateKey(struct pipes_struct *p, struct winreg_CreateKey *r);
struct winreg_DeleteKey;
WERROR _winreg_DeleteKey(struct pipes_struct *p, struct winreg_DeleteKey *r);
struct winreg_DeleteValue;
WERROR _winreg_DeleteValue(struct pipes_struct *p, struct winreg_DeleteValue *r);
struct winreg_EnumKey;
WERROR _winreg_EnumKey(struct pipes_struct *p, struct winreg_EnumKey *r);
struct winreg_EnumValue;
WERROR _winreg_EnumValue(struct pipes_struct *p, struct winreg_EnumValue *r);
struct winreg_FlushKey;
WERROR _winreg_FlushKey(struct pipes_struct *p, struct winreg_FlushKey *r);
struct winreg_GetKeySecurity;
WERROR _winreg_GetKeySecurity(struct pipes_struct *p, struct winreg_GetKeySecurity *r);
struct winreg_LoadKey;
WERROR _winreg_LoadKey(struct pipes_struct *p, struct winreg_LoadKey *r);
struct winreg_NotifyChangeKeyValue;
WERROR _winreg_NotifyChangeKeyValue(struct pipes_struct *p, struct winreg_NotifyChangeKeyValue *r);
struct winreg_OpenKey;
WERROR _winreg_OpenKey(struct pipes_struct *p, struct winreg_OpenKey *r);
struct winreg_QueryInfoKey;
WERROR _winreg_QueryInfoKey(struct pipes_struct *p, struct winreg_QueryInfoKey *r);
struct winreg_QueryValue;
WERROR _winreg_QueryValue(struct pipes_struct *p, struct winreg_QueryValue *r);
struct winreg_ReplaceKey;
WERROR _winreg_ReplaceKey(struct pipes_struct *p, struct winreg_ReplaceKey *r);
struct winreg_RestoreKey;
WERROR _winreg_RestoreKey(struct pipes_struct *p, struct winreg_RestoreKey *r);
struct winreg_SaveKey;
WERROR _winreg_SaveKey(struct pipes_struct *p, struct winreg_SaveKey *r);
struct winreg_SetKeySecurity;
WERROR _winreg_SetKeySecurity(struct pipes_struct *p, struct winreg_SetKeySecurity *r);
struct winreg_SetValue;
WERROR _winreg_SetValue(struct pipes_struct *p, struct winreg_SetValue *r);
struct winreg_UnLoadKey;
WERROR _winreg_UnLoadKey(struct pipes_struct *p, struct winreg_UnLoadKey *r);
struct winreg_InitiateSystemShutdown;
WERROR _winreg_InitiateSystemShutdown(struct pipes_struct *p, struct winreg_InitiateSystemShutdown *r);
struct winreg_AbortSystemShutdown;
WERROR _winreg_AbortSystemShutdown(struct pipes_struct *p, struct winreg_AbortSystemShutdown *r);
struct winreg_GetVersion;
WERROR _winreg_GetVersion(struct pipes_struct *p, struct winreg_GetVersion *r);
struct winreg_OpenHKCC;
WERROR _winreg_OpenHKCC(struct pipes_struct *p, struct winreg_OpenHKCC *r);
struct winreg_OpenHKDD;
WERROR _winreg_OpenHKDD(struct pipes_struct *p, struct winreg_OpenHKDD *r);
struct winreg_QueryMultipleValues;
WERROR _winreg_QueryMultipleValues(struct pipes_struct *p, struct winreg_QueryMultipleValues *r);
struct winreg_InitiateSystemShutdownEx;
WERROR _winreg_InitiateSystemShutdownEx(struct pipes_struct *p, struct winreg_InitiateSystemShutdownEx *r);
struct winreg_SaveKeyEx;
WERROR _winreg_SaveKeyEx(struct pipes_struct *p, struct winreg_SaveKeyEx *r);
struct winreg_OpenHKPT;
WERROR _winreg_OpenHKPT(struct pipes_struct *p, struct winreg_OpenHKPT *r);
struct winreg_OpenHKPN;
WERROR _winreg_OpenHKPN(struct pipes_struct *p, struct winreg_OpenHKPN *r);
struct winreg_QueryMultipleValues2;
WERROR _winreg_QueryMultipleValues2(struct pipes_struct *p, struct winreg_QueryMultipleValues2 *r);
struct winreg_DeleteKeyEx;
WERROR _winreg_DeleteKeyEx(struct pipes_struct *p, struct winreg_DeleteKeyEx *r);
NTSTATUS winreg__op_dispatch(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
NTSTATUS winreg__op_reply(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
NTSTATUS winreg__op_ndr_push(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, struct ndr_push *push, const void *r);
NTSTATUS winreg__op_local(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
#endif /* __NDR_WINREG_SCOMPAT_H__ */
