/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/orpc.h"

#ifndef _HEADER_NDR_ObjectRpcBaseTypes
#define _HEADER_NDR_ObjectRpcBaseTypes

extern const struct ndr_interface_table ndr_table_ObjectRpcBaseTypes;
#define NDR_OBJECTRPCBASETYPES_CALL_COUNT (0)
enum ndr_err_code ndr_push_COMVERSION(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct COMVERSION *r);
enum ndr_err_code ndr_pull_COMVERSION(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct COMVERSION *r);
void ndr_print_COMVERSION(struct ndr_print *ndr, const char *name, const struct COMVERSION *r);
enum ndr_err_code ndr_push_ORPC_EXTENT(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ORPC_EXTENT *r);
enum ndr_err_code ndr_pull_ORPC_EXTENT(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ORPC_EXTENT *r);
void ndr_print_ORPC_EXTENT(struct ndr_print *ndr, const char *name, const struct ORPC_EXTENT *r);
void ndr_print_ORPC_EXTENT_ARRAY(struct ndr_print *ndr, const char *name, const struct ORPC_EXTENT_ARRAY *r);
enum ndr_err_code ndr_push_ORPCTHIS(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ORPCTHIS *r);
enum ndr_err_code ndr_pull_ORPCTHIS(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ORPCTHIS *r);
void ndr_print_ORPCTHIS(struct ndr_print *ndr, const char *name, const struct ORPCTHIS *r);
enum ndr_err_code ndr_push_ORPCTHAT(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ORPCTHAT *r);
enum ndr_err_code ndr_pull_ORPCTHAT(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ORPCTHAT *r);
void ndr_print_ORPCTHAT(struct ndr_print *ndr, const char *name, const struct ORPCTHAT *r);
enum ndr_err_code ndr_push_STRINGBINDING(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct STRINGBINDING *r);
enum ndr_err_code ndr_pull_STRINGBINDING(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct STRINGBINDING *r);
void ndr_print_STRINGBINDING(struct ndr_print *ndr, const char *name, const struct STRINGBINDING *r);
enum ndr_err_code ndr_push_STRINGARRAY(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct STRINGARRAY *r);
enum ndr_err_code ndr_pull_STRINGARRAY(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct STRINGARRAY *r);
void ndr_print_STRINGARRAY(struct ndr_print *ndr, const char *name, const struct STRINGARRAY *r);
enum ndr_err_code ndr_push_DUALSTRINGARRAY(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct DUALSTRINGARRAY *r);
enum ndr_err_code ndr_pull_DUALSTRINGARRAY(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct DUALSTRINGARRAY *r);
void ndr_print_DUALSTRINGARRAY(struct ndr_print *ndr, const char *name, const struct DUALSTRINGARRAY *r);
enum ndr_err_code ndr_push_SECURITYBINDING(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SECURITYBINDING *r);
enum ndr_err_code ndr_pull_SECURITYBINDING(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SECURITYBINDING *r);
void ndr_print_SECURITYBINDING(struct ndr_print *ndr, const char *name, const struct SECURITYBINDING *r);
enum ndr_err_code ndr_push_STDOBJREF(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct STDOBJREF *r);
enum ndr_err_code ndr_pull_STDOBJREF(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct STDOBJREF *r);
void ndr_print_STDOBJREF(struct ndr_print *ndr, const char *name, const struct STDOBJREF *r);
void ndr_print_u_standard(struct ndr_print *ndr, const char *name, const struct u_standard *r);
void ndr_print_u_handler(struct ndr_print *ndr, const char *name, const struct u_handler *r);
void ndr_print_u_custom(struct ndr_print *ndr, const char *name, const struct u_custom *r);
void ndr_print_u_null(struct ndr_print *ndr, const char *name, const struct u_null *r);
void ndr_print_OBJREF_Types(struct ndr_print *ndr, const char *name, const union OBJREF_Types *r);
enum ndr_err_code ndr_push_OBJREF(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct OBJREF *r);
enum ndr_err_code ndr_pull_OBJREF(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct OBJREF *r);
void ndr_print_OBJREF(struct ndr_print *ndr, const char *name, const struct OBJREF *r);
enum ndr_err_code ndr_push_MInterfacePointer(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct MInterfacePointer *r);
enum ndr_err_code ndr_pull_MInterfacePointer(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct MInterfacePointer *r);
void ndr_print_MInterfacePointer(struct ndr_print *ndr, const char *name, const struct MInterfacePointer *r);
enum ndr_err_code ndr_push_COMRESULT(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum COMRESULT r);
enum ndr_err_code ndr_pull_COMRESULT(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum COMRESULT *r);
void ndr_print_COMRESULT(struct ndr_print *ndr, const char *name, enum COMRESULT r);
#endif /* _HEADER_NDR_ObjectRpcBaseTypes */
