/* header auto-generated by pidl */

#ifndef _PIDL_HEADER_rpc_host_msg
#define _PIDL_HEADER_rpc_host_msg

#include <stdint.h>


#include "librpc/gen_ndr/named_pipe_auth.h"
#include "librpc/gen_ndr/dcerpc.h"
#ifndef _HEADER_rpc_host_msg
#define _HEADER_rpc_host_msg

struct rpc_host_client {
	const char *binding;/* [charset(UTF8),unique] */
	struct named_pipe_auth_req_info8 *npa_info8;/* [unique] */
	DATA_BLOB bind_packet;
}/* [public] */;

struct rpc_worker_status {
	uint32_t server_index;
	uint32_t worker_index;
	uint32_t num_association_groups;
	uint32_t num_connections;
}/* [flag(LIBNDR_FLAG_NOALIGN),public] */;

#endif /* _HEADER_rpc_host_msg */
#endif /* _PIDL_HEADER_rpc_host_msg */
