#ifndef __SOURCE4_TORTURE_AUTH_PROTO_H__
#define __SOURCE4_TORTURE_AUTH_PROTO_H__

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
/* This file was automatically generated by mkproto.pl. DO NOT EDIT */

/* this file contains prototypes for functions that are private 
 * to this subsystem or library. These functions should not be 
 * used outside this particular subsystem! */


/* The following definitions come from ../../source4/torture/auth/ntlmssp.c  */

struct torture_suite *torture_ntlmssp(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/auth/pac.c  */

struct torture_suite *torture_pac(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/auth/smbencrypt.c  */

struct torture_suite *torture_smbencrypt(TALLOC_CTX *mem_ctx);
#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2)

#endif /* __SOURCE4_TORTURE_AUTH_PROTO_H__ */

