/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/smb3posix.h"

#ifndef _HEADER_NDR_smb3posix
#define _HEADER_NDR_smb3posix

extern const struct ndr_interface_table ndr_table_smb3posix;
#define NDR_SMB3POSIX_CALL_COUNT (0)
enum ndr_err_code ndr_push_smb3_posix_cc_info(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct smb3_posix_cc_info *r);
enum ndr_err_code ndr_pull_smb3_posix_cc_info(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct smb3_posix_cc_info *r);
void ndr_print_smb3_posix_cc_info(struct ndr_print *ndr, const char *name, const struct smb3_posix_cc_info *r);
enum ndr_err_code ndr_push_smb3_file_posix_information(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct smb3_file_posix_information *r);
enum ndr_err_code ndr_pull_smb3_file_posix_information(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct smb3_file_posix_information *r);
void ndr_print_smb3_file_posix_information(struct ndr_print *ndr, const char *name, const struct smb3_file_posix_information *r);
#endif /* _HEADER_NDR_smb3posix */
