/* header auto-generated by pidl */

#ifndef _PIDL_HEADER_fscc
#define _PIDL_HEADER_fscc

#include <stdint.h>


#include "librpc/gen_ndr/misc.h"
#ifndef _HEADER_fscc
#define _HEADER_fscc

struct fscc_FileObjectIdBuffer_2 {
	struct GUID id;
	struct GUID birthVolumeId;
	struct GUID initialObjectId;
	struct GUID domainId;
}/* [public] */;

/* bitmap fscc_FileAttributes */
#define FSCC_FILE_ATTRIBUTE_READONLY ( 0x00000001 )
#define FSCC_FILE_ATTRIBUTE_HIDDEN ( 0x00000002 )
#define FSCC_FILE_ATTRIBUTE_SYSTEM ( 0x00000004 )
#define FSCC_FILE_ATTRIBUTE_NORMAL ( 0x00000008 )
#define FSCC_FILE_ATTRIBUTE_DIRECTORY ( 0x00000010 )
#define FSCC_FILE_ATTRIBUTE_ARCHIVE ( 0x00000020 )
#define FSCC_FILE_ATTRIBUTE_TEMPORARY ( 0x00000100 )
#define FSCC_FILE_ATTRIBUTE_SPARSE_FILE ( 0x00000200 )
#define FSCC_FILE_ATTRIBUTE_REPARSE_POINT ( 0x00000400 )
#define FSCC_FILE_ATTRIBUTE_COMPRESSED ( 0x00000800 )
#define FSCC_FILE_ATTRIBUTE_OFFLINE ( 0x00001000 )
#define FSCC_FILE_ATTRIBUTE_NOT_CONTENT_INDEXED ( 0x00002000 )
#define FSCC_FILE_ATTRIBUTE_ENCRYPTED ( 0x00004000 )

struct fscc_FileNetworkOpenInformation {
	uint64_t creationTime;
	uint64_t lastAccessTime;
	uint64_t lastWriteTime;
	uint64_t changeTime;
	uint64_t allocSize;
	uint64_t endOfFile;
	uint32_t fileAttribute;
	uint32_t reserved;
}/* [public] */;

#endif /* _HEADER_fscc */
#endif /* _PIDL_HEADER_fscc */
