#ifndef __SOURCE4_NBT_SERVER_WINS_WINSDB_PROTO_H__
#define __SOURCE4_NBT_SERVER_WINS_WINSDB_PROTO_H__

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
/* This file was automatically generated by mkproto.pl. DO NOT EDIT */

/* this file contains prototypes for functions that are private 
 * to this subsystem or library. These functions should not be 
 * used outside this particular subsystem! */


/* The following definitions come from ../../source4/nbt_server/wins/winsdb.c  */

uint64_t winsdb_get_maxVersion(struct winsdb_handle *h);
uint64_t winsdb_set_maxVersion(struct winsdb_handle *h, uint64_t newMaxVersion);
struct winsdb_addr **winsdb_addr_list_make(TALLOC_CTX *mem_ctx);
struct winsdb_addr **winsdb_addr_list_add(struct winsdb_handle *h, const struct winsdb_record *rec,
					  struct winsdb_addr **addresses, const char *address,
					  const char *wins_owner, time_t expire_time,
					  bool is_name_registration);
void winsdb_addr_list_remove(struct winsdb_addr **addresses, const char *address);
struct winsdb_addr *winsdb_addr_list_check(struct winsdb_addr **addresses, const char *address);
size_t winsdb_addr_list_length(struct winsdb_addr **addresses);
const char **winsdb_addr_string_list(TALLOC_CTX *mem_ctx, struct winsdb_addr **addresses);
NTSTATUS winsdb_lookup(struct winsdb_handle *h, 
		       const struct nbt_name *name,
		       TALLOC_CTX *mem_ctx,
		       struct winsdb_record **_rec);
NTSTATUS winsdb_record(struct winsdb_handle *h, struct ldb_message *msg, TALLOC_CTX *mem_ctx, time_t now, struct winsdb_record **_rec);
uint8_t winsdb_add(struct winsdb_handle *h, struct winsdb_record *rec, uint32_t flags);
uint8_t winsdb_modify(struct winsdb_handle *h, struct winsdb_record *rec, uint32_t flags);
uint8_t winsdb_delete(struct winsdb_handle *h, struct winsdb_record *rec);
struct winsdb_handle *winsdb_connect(TALLOC_CTX *mem_ctx, 
				     struct tevent_context *ev_ctx,
				     struct loadparm_context *lp_ctx,
				     const char *owner,
				     enum winsdb_handle_caller caller);

/* The following definitions come from ../../source4/nbt_server/wins/wins_hook.c  */

void wins_hook(struct winsdb_handle *h, const struct winsdb_record *rec, 
	       enum wins_hook_action action, const char *wins_hook_script);
#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2)

#endif /* __SOURCE4_NBT_SERVER_WINS_WINSDB_PROTO_H__ */

