#ifndef __NDR_SRVSVC_SCOMPAT_H__
#define __NDR_SRVSVC_SCOMPAT_H__

struct pipes_struct;
struct dcesrv_endpoint_server;
struct dcesrv_call_state;

const struct dcesrv_endpoint_server *srvsvc_get_ep_server(void);

NTSTATUS srvsvc__op_ndr_pull(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, struct ndr_pull *pull, void **r);
struct srvsvc_NetCharDevEnum;
WERROR _srvsvc_NetCharDevEnum(struct pipes_struct *p, struct srvsvc_NetCharDevEnum *r);
struct srvsvc_NetCharDevGetInfo;
WERROR _srvsvc_NetCharDevGetInfo(struct pipes_struct *p, struct srvsvc_NetCharDevGetInfo *r);
struct srvsvc_NetCharDevControl;
WERROR _srvsvc_NetCharDevControl(struct pipes_struct *p, struct srvsvc_NetCharDevControl *r);
struct srvsvc_NetCharDevQEnum;
WERROR _srvsvc_NetCharDevQEnum(struct pipes_struct *p, struct srvsvc_NetCharDevQEnum *r);
struct srvsvc_NetCharDevQGetInfo;
WERROR _srvsvc_NetCharDevQGetInfo(struct pipes_struct *p, struct srvsvc_NetCharDevQGetInfo *r);
struct srvsvc_NetCharDevQSetInfo;
WERROR _srvsvc_NetCharDevQSetInfo(struct pipes_struct *p, struct srvsvc_NetCharDevQSetInfo *r);
struct srvsvc_NetCharDevQPurge;
WERROR _srvsvc_NetCharDevQPurge(struct pipes_struct *p, struct srvsvc_NetCharDevQPurge *r);
struct srvsvc_NetCharDevQPurgeSelf;
WERROR _srvsvc_NetCharDevQPurgeSelf(struct pipes_struct *p, struct srvsvc_NetCharDevQPurgeSelf *r);
struct srvsvc_NetConnEnum;
WERROR _srvsvc_NetConnEnum(struct pipes_struct *p, struct srvsvc_NetConnEnum *r);
struct srvsvc_NetFileEnum;
WERROR _srvsvc_NetFileEnum(struct pipes_struct *p, struct srvsvc_NetFileEnum *r);
struct srvsvc_NetFileGetInfo;
WERROR _srvsvc_NetFileGetInfo(struct pipes_struct *p, struct srvsvc_NetFileGetInfo *r);
struct srvsvc_NetFileClose;
WERROR _srvsvc_NetFileClose(struct pipes_struct *p, struct srvsvc_NetFileClose *r);
struct srvsvc_NetSessEnum;
WERROR _srvsvc_NetSessEnum(struct pipes_struct *p, struct srvsvc_NetSessEnum *r);
struct srvsvc_NetSessDel;
WERROR _srvsvc_NetSessDel(struct pipes_struct *p, struct srvsvc_NetSessDel *r);
struct srvsvc_NetShareAdd;
WERROR _srvsvc_NetShareAdd(struct pipes_struct *p, struct srvsvc_NetShareAdd *r);
struct srvsvc_NetShareEnumAll;
WERROR _srvsvc_NetShareEnumAll(struct pipes_struct *p, struct srvsvc_NetShareEnumAll *r);
struct srvsvc_NetShareGetInfo;
WERROR _srvsvc_NetShareGetInfo(struct pipes_struct *p, struct srvsvc_NetShareGetInfo *r);
struct srvsvc_NetShareSetInfo;
WERROR _srvsvc_NetShareSetInfo(struct pipes_struct *p, struct srvsvc_NetShareSetInfo *r);
struct srvsvc_NetShareDel;
WERROR _srvsvc_NetShareDel(struct pipes_struct *p, struct srvsvc_NetShareDel *r);
struct srvsvc_NetShareDelSticky;
WERROR _srvsvc_NetShareDelSticky(struct pipes_struct *p, struct srvsvc_NetShareDelSticky *r);
struct srvsvc_NetShareCheck;
WERROR _srvsvc_NetShareCheck(struct pipes_struct *p, struct srvsvc_NetShareCheck *r);
struct srvsvc_NetSrvGetInfo;
WERROR _srvsvc_NetSrvGetInfo(struct pipes_struct *p, struct srvsvc_NetSrvGetInfo *r);
struct srvsvc_NetSrvSetInfo;
WERROR _srvsvc_NetSrvSetInfo(struct pipes_struct *p, struct srvsvc_NetSrvSetInfo *r);
struct srvsvc_NetDiskEnum;
WERROR _srvsvc_NetDiskEnum(struct pipes_struct *p, struct srvsvc_NetDiskEnum *r);
struct srvsvc_NetServerStatisticsGet;
WERROR _srvsvc_NetServerStatisticsGet(struct pipes_struct *p, struct srvsvc_NetServerStatisticsGet *r);
struct srvsvc_NetTransportAdd;
WERROR _srvsvc_NetTransportAdd(struct pipes_struct *p, struct srvsvc_NetTransportAdd *r);
struct srvsvc_NetTransportEnum;
WERROR _srvsvc_NetTransportEnum(struct pipes_struct *p, struct srvsvc_NetTransportEnum *r);
struct srvsvc_NetTransportDel;
WERROR _srvsvc_NetTransportDel(struct pipes_struct *p, struct srvsvc_NetTransportDel *r);
struct srvsvc_NetRemoteTOD;
WERROR _srvsvc_NetRemoteTOD(struct pipes_struct *p, struct srvsvc_NetRemoteTOD *r);
struct srvsvc_NetSetServiceBits;
WERROR _srvsvc_NetSetServiceBits(struct pipes_struct *p, struct srvsvc_NetSetServiceBits *r);
struct srvsvc_NetPathType;
WERROR _srvsvc_NetPathType(struct pipes_struct *p, struct srvsvc_NetPathType *r);
struct srvsvc_NetPathCanonicalize;
WERROR _srvsvc_NetPathCanonicalize(struct pipes_struct *p, struct srvsvc_NetPathCanonicalize *r);
struct srvsvc_NetPathCompare;
WERROR _srvsvc_NetPathCompare(struct pipes_struct *p, struct srvsvc_NetPathCompare *r);
struct srvsvc_NetNameValidate;
WERROR _srvsvc_NetNameValidate(struct pipes_struct *p, struct srvsvc_NetNameValidate *r);
struct srvsvc_NETRPRNAMECANONICALIZE;
WERROR _srvsvc_NETRPRNAMECANONICALIZE(struct pipes_struct *p, struct srvsvc_NETRPRNAMECANONICALIZE *r);
struct srvsvc_NetPRNameCompare;
WERROR _srvsvc_NetPRNameCompare(struct pipes_struct *p, struct srvsvc_NetPRNameCompare *r);
struct srvsvc_NetShareEnum;
WERROR _srvsvc_NetShareEnum(struct pipes_struct *p, struct srvsvc_NetShareEnum *r);
struct srvsvc_NetShareDelStart;
WERROR _srvsvc_NetShareDelStart(struct pipes_struct *p, struct srvsvc_NetShareDelStart *r);
struct srvsvc_NetShareDelCommit;
WERROR _srvsvc_NetShareDelCommit(struct pipes_struct *p, struct srvsvc_NetShareDelCommit *r);
struct srvsvc_NetGetFileSecurity;
WERROR _srvsvc_NetGetFileSecurity(struct pipes_struct *p, struct srvsvc_NetGetFileSecurity *r);
struct srvsvc_NetSetFileSecurity;
WERROR _srvsvc_NetSetFileSecurity(struct pipes_struct *p, struct srvsvc_NetSetFileSecurity *r);
struct srvsvc_NetServerTransportAddEx;
WERROR _srvsvc_NetServerTransportAddEx(struct pipes_struct *p, struct srvsvc_NetServerTransportAddEx *r);
struct srvsvc_NetServerSetServiceBitsEx;
WERROR _srvsvc_NetServerSetServiceBitsEx(struct pipes_struct *p, struct srvsvc_NetServerSetServiceBitsEx *r);
struct srvsvc_NETRDFSGETVERSION;
WERROR _srvsvc_NETRDFSGETVERSION(struct pipes_struct *p, struct srvsvc_NETRDFSGETVERSION *r);
struct srvsvc_NETRDFSCREATELOCALPARTITION;
WERROR _srvsvc_NETRDFSCREATELOCALPARTITION(struct pipes_struct *p, struct srvsvc_NETRDFSCREATELOCALPARTITION *r);
struct srvsvc_NETRDFSDELETELOCALPARTITION;
WERROR _srvsvc_NETRDFSDELETELOCALPARTITION(struct pipes_struct *p, struct srvsvc_NETRDFSDELETELOCALPARTITION *r);
struct srvsvc_NETRDFSSETLOCALVOLUMESTATE;
WERROR _srvsvc_NETRDFSSETLOCALVOLUMESTATE(struct pipes_struct *p, struct srvsvc_NETRDFSSETLOCALVOLUMESTATE *r);
struct srvsvc_NETRDFSSETSERVERINFO;
WERROR _srvsvc_NETRDFSSETSERVERINFO(struct pipes_struct *p, struct srvsvc_NETRDFSSETSERVERINFO *r);
struct srvsvc_NETRDFSCREATEEXITPOINT;
WERROR _srvsvc_NETRDFSCREATEEXITPOINT(struct pipes_struct *p, struct srvsvc_NETRDFSCREATEEXITPOINT *r);
struct srvsvc_NETRDFSDELETEEXITPOINT;
WERROR _srvsvc_NETRDFSDELETEEXITPOINT(struct pipes_struct *p, struct srvsvc_NETRDFSDELETEEXITPOINT *r);
struct srvsvc_NETRDFSMODIFYPREFIX;
WERROR _srvsvc_NETRDFSMODIFYPREFIX(struct pipes_struct *p, struct srvsvc_NETRDFSMODIFYPREFIX *r);
struct srvsvc_NETRDFSFIXLOCALVOLUME;
WERROR _srvsvc_NETRDFSFIXLOCALVOLUME(struct pipes_struct *p, struct srvsvc_NETRDFSFIXLOCALVOLUME *r);
struct srvsvc_NETRDFSMANAGERREPORTSITEINFO;
WERROR _srvsvc_NETRDFSMANAGERREPORTSITEINFO(struct pipes_struct *p, struct srvsvc_NETRDFSMANAGERREPORTSITEINFO *r);
struct srvsvc_NETRSERVERTRANSPORTDELEX;
WERROR _srvsvc_NETRSERVERTRANSPORTDELEX(struct pipes_struct *p, struct srvsvc_NETRSERVERTRANSPORTDELEX *r);
NTSTATUS srvsvc__op_dispatch(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
NTSTATUS srvsvc__op_reply(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
NTSTATUS srvsvc__op_ndr_push(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, struct ndr_push *push, const void *r);
NTSTATUS srvsvc__op_local(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
#endif /* __NDR_SRVSVC_SCOMPAT_H__ */
