/* header auto-generated by pidl */

#ifndef _PIDL_HEADER_policyagent
#define _PIDL_HEADER_policyagent

#include <stdint.h>


#ifndef _HEADER_policyagent
#define _HEADER_policyagent


struct policyagent_Dummy {
	struct {
		WERROR result;
	} out;

};

#endif /* _HEADER_policyagent */
#endif /* _PIDL_HEADER_policyagent */
