/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/ntprinting.h"

#ifndef _HEADER_NDR_ntprinting
#define _HEADER_NDR_ntprinting

#include "../librpc/ndr/ndr_ntprinting.h"
#define NDR_NTPRINTING_UUID "a025d3cb-c605-40d6-86e1-4cff18e7dd94"
#define NDR_NTPRINTING_VERSION 0.0
#define NDR_NTPRINTING_NAME "ntprinting"
#define NDR_NTPRINTING_HELPSTRING "s3 printing tdb formats"
extern const struct ndr_interface_table ndr_table_ntprinting;
#define NDR_NTPRINTING_CALL_COUNT (0)
enum ndr_err_code ndr_push_ntprinting_form(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ntprinting_form *r);
enum ndr_err_code ndr_pull_ntprinting_form(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ntprinting_form *r);
void ndr_print_ntprinting_form(struct ndr_print *ndr, const char *name, const struct ntprinting_form *r);
enum ndr_err_code ndr_push_ntprinting_driver(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ntprinting_driver *r);
enum ndr_err_code ndr_pull_ntprinting_driver(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ntprinting_driver *r);
void ndr_print_ntprinting_driver(struct ndr_print *ndr, const char *name, const struct ntprinting_driver *r);
enum ndr_err_code ndr_push_ntprinting_devicemode(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ntprinting_devicemode *r);
enum ndr_err_code ndr_pull_ntprinting_devicemode(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ntprinting_devicemode *r);
void ndr_print_ntprinting_devicemode(struct ndr_print *ndr, const char *name, const struct ntprinting_devicemode *r);
enum ndr_err_code ndr_push_ntprinting_printer_data(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ntprinting_printer_data *r);
enum ndr_err_code ndr_pull_ntprinting_printer_data(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ntprinting_printer_data *r);
void ndr_print_ntprinting_printer_data(struct ndr_print *ndr, const char *name, const struct ntprinting_printer_data *r);
enum ndr_err_code ndr_push_ntprinting_printer_info(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ntprinting_printer_info *r);
enum ndr_err_code ndr_pull_ntprinting_printer_info(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ntprinting_printer_info *r);
void ndr_print_ntprinting_printer_info(struct ndr_print *ndr, const char *name, const struct ntprinting_printer_info *r);
enum ndr_err_code ndr_push_ntprinting_printer(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ntprinting_printer *r);
enum ndr_err_code ndr_pull_ntprinting_printer(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ntprinting_printer *r);
void ndr_print_ntprinting_printer(struct ndr_print *ndr, const char *name, const struct ntprinting_printer *r);
#endif /* _HEADER_NDR_ntprinting */
