#ifndef __SOURCE4_LIBNET_LIBNET_JOIN_PROTO_H__
#define __SOURCE4_LIBNET_LIBNET_JOIN_PROTO_H__

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
/* This file was automatically generated by mkproto.pl. DO NOT EDIT */

/* this file contains prototypes for functions that are private 
 * to this subsystem or library. These functions should not be 
 * used outside this particular subsystem! */


/* The following definitions come from ../../source4/libnet/libnet_join.c  */

NTSTATUS libnet_JoinDomain(struct libnet_context *ctx, TALLOC_CTX *mem_ctx, struct libnet_JoinDomain *r);
NTSTATUS libnet_Join_member(struct libnet_context *ctx,
			    TALLOC_CTX *mem_ctx,
			    struct libnet_Join_member *r);

/* The following definitions come from ../../source4/libnet/libnet_vampire.c  */

void *libnet_vampire_replicate_init(TALLOC_CTX *mem_ctx,
				    struct ldb_context *samdb,
				    struct loadparm_context *lp_ctx);
void *libnet_vampire_cb_state_init(TALLOC_CTX *mem_ctx,
				   struct loadparm_context *lp_ctx, struct tevent_context *event_ctx,
				   const char *netbios_name, const char *domain_name, const char *realm,
				   const char *targetdir);
struct ldb_context *libnet_vampire_cb_ldb(struct libnet_vampire_cb_state *state);
struct loadparm_context *libnet_vampire_cb_lp_ctx(struct libnet_vampire_cb_state *state);
NTSTATUS libnet_vampire_cb_prepare_db(void *private_data,
				      const struct libnet_BecomeDC_PrepareDB *p);
NTSTATUS libnet_vampire_cb_check_options(void *private_data,
					 const struct libnet_BecomeDC_CheckOptions *o);
WERROR libnet_vampire_cb_schema_chunk(void *private_data,
				      const struct libnet_BecomeDC_StoreChunk *c);
WERROR libnet_vampire_cb_store_chunk(void *private_data,
				     const struct libnet_BecomeDC_StoreChunk *c);
#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2)

#endif /* __SOURCE4_LIBNET_LIBNET_JOIN_PROTO_H__ */

