/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/source3/librpc/gen_ndr/secrets.h"

#ifndef _HEADER_NDR_secrets
#define _HEADER_NDR_secrets

extern const struct ndr_interface_table ndr_table_secrets;
#define NDR_SECRETS_CALL_COUNT (0)
enum ndr_err_code ndr_push_TRUSTED_DOM_PASS(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct TRUSTED_DOM_PASS *r);
enum ndr_err_code ndr_pull_TRUSTED_DOM_PASS(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct TRUSTED_DOM_PASS *r);
void ndr_print_TRUSTED_DOM_PASS(struct ndr_print *ndr, const char *name, const struct TRUSTED_DOM_PASS *r);
enum ndr_err_code ndr_push_lsa_secret(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct lsa_secret *r);
enum ndr_err_code ndr_pull_lsa_secret(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct lsa_secret *r);
void ndr_print_lsa_secret(struct ndr_print *ndr, const char *name, const struct lsa_secret *r);
void ndr_print_secrets_domain_info1_kerberos_key(struct ndr_print *ndr, const char *name, const struct secrets_domain_info1_kerberos_key *r);
void ndr_print_secrets_domain_info1_password(struct ndr_print *ndr, const char *name, const struct secrets_domain_info1_password *r);
void ndr_print_secrets_domain_info1_change(struct ndr_print *ndr, const char *name, const struct secrets_domain_info1_change *r);
enum ndr_err_code ndr_push_secrets_domain_info1(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct secrets_domain_info1 *r);
enum ndr_err_code ndr_pull_secrets_domain_info1(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct secrets_domain_info1 *r);
void ndr_print_secrets_domain_info1(struct ndr_print *ndr, const char *name, const struct secrets_domain_info1 *r);
void ndr_print_secrets_domain_info_version(struct ndr_print *ndr, const char *name, enum secrets_domain_info_version r);
void ndr_print_secrets_domain_infoU(struct ndr_print *ndr, const char *name, const union secrets_domain_infoU *r);
enum ndr_err_code ndr_push_secrets_domain_infoB(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct secrets_domain_infoB *r);
enum ndr_err_code ndr_pull_secrets_domain_infoB(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct secrets_domain_infoB *r);
void ndr_print_secrets_domain_infoB(struct ndr_print *ndr, const char *name, const struct secrets_domain_infoB *r);
#endif /* _HEADER_NDR_secrets */
