/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/notify.h"

#ifndef _HEADER_NDR_notify
#define _HEADER_NDR_notify

extern const struct ndr_interface_table ndr_table_notify;
#define NDR_NOTIFY_CALL_COUNT (0)
enum ndr_err_code ndr_push_notify_entry(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct notify_entry *r);
enum ndr_err_code ndr_pull_notify_entry(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct notify_entry *r);
void ndr_print_notify_entry(struct ndr_print *ndr, const char *name, const struct notify_entry *r);
enum ndr_err_code ndr_push_notify_entry_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct notify_entry_array *r);
enum ndr_err_code ndr_pull_notify_entry_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct notify_entry_array *r);
void ndr_print_notify_entry_array(struct ndr_print *ndr, const char *name, const struct notify_entry_array *r);
enum ndr_err_code ndr_push_notify_db_entry(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct notify_db_entry *r);
enum ndr_err_code ndr_pull_notify_db_entry(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct notify_db_entry *r);
void ndr_print_notify_db_entry(struct ndr_print *ndr, const char *name, const struct notify_db_entry *r);
void ndr_print_notify_depth(struct ndr_print *ndr, const char *name, const struct notify_depth *r);
enum ndr_err_code ndr_push_notify_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct notify_array *r);
enum ndr_err_code ndr_pull_notify_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct notify_array *r);
void ndr_print_notify_array(struct ndr_print *ndr, const char *name, const struct notify_array *r);
enum ndr_err_code ndr_push_notify_event(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct notify_event *r);
enum ndr_err_code ndr_pull_notify_event(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct notify_event *r);
void ndr_print_notify_event(struct ndr_print *ndr, const char *name, const struct notify_event *r);
void ndr_print_FILE_NOTIFY_ACTION(struct ndr_print *ndr, const char *name, enum FILE_NOTIFY_ACTION r);
enum ndr_err_code ndr_push_FILE_NOTIFY_INFORMATION(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct FILE_NOTIFY_INFORMATION *r);
enum ndr_err_code ndr_pull_FILE_NOTIFY_INFORMATION(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct FILE_NOTIFY_INFORMATION *r);
void ndr_print_FILE_NOTIFY_INFORMATION(struct ndr_print *ndr, const char *name, const struct FILE_NOTIFY_INFORMATION *r);
size_t ndr_size_FILE_NOTIFY_INFORMATION(const struct FILE_NOTIFY_INFORMATION *r, libndr_flags flags);
#endif /* _HEADER_NDR_notify */
