/* header auto-generated by pidl */

#ifndef _PIDL_HEADER_file_quota
#define _PIDL_HEADER_file_quota

#include <stdint.h>


#include "librpc/gen_ndr/security.h"
#ifndef _HEADER_file_quota
#define _HEADER_file_quota

struct file_get_quota_info {
	uint32_t next_entry_offset;
	uint32_t sid_length;
	struct dom_sid sid;
}/* [public] */;

struct file_quota_information {
	uint32_t next_entry_offset;
	uint32_t sid_length;
	uint64_t change_time;
	uint64_t quota_used;
	uint64_t quota_threshold;
	uint64_t quota_limit;
	struct dom_sid sid;
}/* [public] */;

#endif /* _HEADER_file_quota */
#ifndef _HEADER_smb2_query_quota
#define _HEADER_smb2_query_quota

struct smb2_query_quota_info {
	uint8_t return_single;
	uint8_t restart_scan;
	uint16_t reserved;
	uint32_t sid_list_length;
	uint32_t start_sid_length;
	uint32_t start_sid_offset;
}/* [public] */;

#endif /* _HEADER_smb2_query_quota */
#ifndef _HEADER_smb1_nt_transact_query_quota
#define _HEADER_smb1_nt_transact_query_quota

struct nttrans_query_quota_params {
	uint16_t fid;
	uint8_t return_single_entry;
	uint8_t restart_scan;
	uint32_t sid_list_length;
	uint32_t start_sid_length;
	uint32_t start_sid_offset;
}/* [public] */;

#endif /* _HEADER_smb1_nt_transact_query_quota */
#endif /* _PIDL_HEADER_file_quota */
