/* header auto-generated by pidl */

#ifndef _PIDL_HEADER_msgsvc
#define _PIDL_HEADER_msgsvc

#include <stdint.h>


#ifndef _HEADER_msgsvc
#define _HEADER_msgsvc


struct NetrMessageNameAdd {
	int _dummy_element;
};


struct NetrMessageNameEnum {
	int _dummy_element;
};


struct NetrMessageNameGetInfo {
	int _dummy_element;
};


struct NetrMessageNameDel {
	int _dummy_element;
};

#endif /* _HEADER_msgsvc */
#ifndef _HEADER_msgsvcsend
#define _HEADER_msgsvcsend


struct NetrSendMessage {
	int _dummy_element;
};

#endif /* _HEADER_msgsvcsend */
#endif /* _PIDL_HEADER_msgsvc */
