/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/spoolss.h"

#ifndef _HEADER_NDR_spoolss
#define _HEADER_NDR_spoolss

#include "../librpc/ndr/ndr_spoolss_buf.h"
#define NDR_SPOOLSS_UUID "12345678-1234-abcd-ef00-0123456789ab"
#define NDR_SPOOLSS_VERSION 1.0
#define NDR_SPOOLSS_NAME "spoolss"
#define NDR_SPOOLSS_HELPSTRING "Spooler SubSystem"
extern const struct ndr_interface_table ndr_table_spoolss;
#define NDR_SPOOLSS_ENUMPRINTERS (0x00)

#define NDR_SPOOLSS_OPENPRINTER (0x01)

#define NDR_SPOOLSS_SETJOB (0x02)

#define NDR_SPOOLSS_GETJOB (0x03)

#define NDR_SPOOLSS_ENUMJOBS (0x04)

#define NDR_SPOOLSS_ADDPRINTER (0x05)

#define NDR_SPOOLSS_DELETEPRINTER (0x06)

#define NDR_SPOOLSS_SETPRINTER (0x07)

#define NDR_SPOOLSS_GETPRINTER (0x08)

#define NDR_SPOOLSS_ADDPRINTERDRIVER (0x09)

#define NDR_SPOOLSS_ENUMPRINTERDRIVERS (0x0a)

#define NDR_SPOOLSS_GETPRINTERDRIVER (0x0b)

#define NDR_SPOOLSS_GETPRINTERDRIVERDIRECTORY (0x0c)

#define NDR_SPOOLSS_DELETEPRINTERDRIVER (0x0d)

#define NDR_SPOOLSS_ADDPRINTPROCESSOR (0x0e)

#define NDR_SPOOLSS_ENUMPRINTPROCESSORS (0x0f)

#define NDR_SPOOLSS_GETPRINTPROCESSORDIRECTORY (0x10)

#define NDR_SPOOLSS_STARTDOCPRINTER (0x11)

#define NDR_SPOOLSS_STARTPAGEPRINTER (0x12)

#define NDR_SPOOLSS_WRITEPRINTER (0x13)

#define NDR_SPOOLSS_ENDPAGEPRINTER (0x14)

#define NDR_SPOOLSS_ABORTPRINTER (0x15)

#define NDR_SPOOLSS_READPRINTER (0x16)

#define NDR_SPOOLSS_ENDDOCPRINTER (0x17)

#define NDR_SPOOLSS_ADDJOB (0x18)

#define NDR_SPOOLSS_SCHEDULEJOB (0x19)

#define NDR_SPOOLSS_GETPRINTERDATA (0x1a)

#define NDR_SPOOLSS_SETPRINTERDATA (0x1b)

#define NDR_SPOOLSS_WAITFORPRINTERCHANGE (0x1c)

#define NDR_SPOOLSS_CLOSEPRINTER (0x1d)

#define NDR_SPOOLSS_ADDFORM (0x1e)

#define NDR_SPOOLSS_DELETEFORM (0x1f)

#define NDR_SPOOLSS_GETFORM (0x20)

#define NDR_SPOOLSS_SETFORM (0x21)

#define NDR_SPOOLSS_ENUMFORMS (0x22)

#define NDR_SPOOLSS_ENUMPORTS (0x23)

#define NDR_SPOOLSS_ENUMMONITORS (0x24)

#define NDR_SPOOLSS_ADDPORT (0x25)

#define NDR_SPOOLSS_CONFIGUREPORT (0x26)

#define NDR_SPOOLSS_DELETEPORT (0x27)

#define NDR_SPOOLSS_CREATEPRINTERIC (0x28)

#define NDR_SPOOLSS_PLAYGDISCRIPTONPRINTERIC (0x29)

#define NDR_SPOOLSS_DELETEPRINTERIC (0x2a)

#define NDR_SPOOLSS_ADDPRINTERCONNECTION (0x2b)

#define NDR_SPOOLSS_DELETEPRINTERCONNECTION (0x2c)

#define NDR_SPOOLSS_PRINTERMESSAGEBOX (0x2d)

#define NDR_SPOOLSS_ADDMONITOR (0x2e)

#define NDR_SPOOLSS_DELETEMONITOR (0x2f)

#define NDR_SPOOLSS_DELETEPRINTPROCESSOR (0x30)

#define NDR_SPOOLSS_ADDPRINTPROVIDOR (0x31)

#define NDR_SPOOLSS_DELETEPRINTPROVIDOR (0x32)

#define NDR_SPOOLSS_ENUMPRINTPROCESSORDATATYPES (0x33)

#define NDR_SPOOLSS_RESETPRINTER (0x34)

#define NDR_SPOOLSS_GETPRINTERDRIVER2 (0x35)

#define NDR_SPOOLSS_FINDFIRSTPRINTERCHANGENOTIFICATION (0x36)

#define NDR_SPOOLSS_FINDNEXTPRINTERCHANGENOTIFICATION (0x37)

#define NDR_SPOOLSS_FINDCLOSEPRINTERNOTIFY (0x38)

#define NDR_SPOOLSS_ROUTERFINDFIRSTPRINTERCHANGENOTIFICATIONOLD (0x39)

#define NDR_SPOOLSS_REPLYOPENPRINTER (0x3a)

#define NDR_SPOOLSS_ROUTERREPLYPRINTER (0x3b)

#define NDR_SPOOLSS_REPLYCLOSEPRINTER (0x3c)

#define NDR_SPOOLSS_ADDPORTEX (0x3d)

#define NDR_SPOOLSS_ROUTERFINDFIRSTPRINTERCHANGENOTIFICATION (0x3e)

#define NDR_SPOOLSS_SPOOLERINIT (0x3f)

#define NDR_SPOOLSS_RESETPRINTEREX (0x40)

#define NDR_SPOOLSS_REMOTEFINDFIRSTPRINTERCHANGENOTIFYEX (0x41)

#define NDR_SPOOLSS_ROUTERREPLYPRINTEREX (0x42)

#define NDR_SPOOLSS_ROUTERREFRESHPRINTERCHANGENOTIFY (0x43)

#define NDR_SPOOLSS_44 (0x44)

#define NDR_SPOOLSS_OPENPRINTEREX (0x45)

#define NDR_SPOOLSS_ADDPRINTEREX (0x46)

#define NDR_SPOOLSS_SETPORT (0x47)

#define NDR_SPOOLSS_ENUMPRINTERDATA (0x48)

#define NDR_SPOOLSS_DELETEPRINTERDATA (0x49)

#define NDR_SPOOLSS_4A (0x4a)

#define NDR_SPOOLSS_4B (0x4b)

#define NDR_SPOOLSS_4C (0x4c)

#define NDR_SPOOLSS_SETPRINTERDATAEX (0x4d)

#define NDR_SPOOLSS_GETPRINTERDATAEX (0x4e)

#define NDR_SPOOLSS_ENUMPRINTERDATAEX (0x4f)

#define NDR_SPOOLSS_ENUMPRINTERKEY (0x50)

#define NDR_SPOOLSS_DELETEPRINTERDATAEX (0x51)

#define NDR_SPOOLSS_DELETEPRINTERKEY (0x52)

#define NDR_SPOOLSS_53 (0x53)

#define NDR_SPOOLSS_DELETEPRINTERDRIVEREX (0x54)

#define NDR_SPOOLSS_ADDPERMACHINECONNECTION (0x55)

#define NDR_SPOOLSS_DELETEPERMACHINECONNECTION (0x56)

#define NDR_SPOOLSS_ENUMPERMACHINECONNECTIONS (0x57)

#define NDR_SPOOLSS_XCVDATA (0x58)

#define NDR_SPOOLSS_ADDPRINTERDRIVEREX (0x59)

#define NDR_SPOOLSS_5A (0x5a)

#define NDR_SPOOLSS_5B (0x5b)

#define NDR_SPOOLSS_5C (0x5c)

#define NDR_SPOOLSS_5D (0x5d)

#define NDR_SPOOLSS_5E (0x5e)

#define NDR_SPOOLSS_5F (0x5f)

#define NDR_SPOOLSS_60 (0x60)

#define NDR_SPOOLSS_SENDRECVBIDIDATA (0x61)

#define NDR_SPOOLSS_62 (0x62)

#define NDR_SPOOLSS_63 (0x63)

#define NDR_SPOOLSS_64 (0x64)

#define NDR_SPOOLSS_65 (0x65)

#define NDR_SPOOLSS_GETCOREPRINTERDRIVERS (0x66)

#define NDR_SPOOLSS_67 (0x67)

#define NDR_SPOOLSS_GETPRINTERDRIVERPACKAGEPATH (0x68)

#define NDR_SPOOLSS_69 (0x69)

#define NDR_SPOOLSS_6A (0x6a)

#define NDR_SPOOLSS_6B (0x6b)

#define NDR_SPOOLSS_6C (0x6c)

#define NDR_SPOOLSS_6D (0x6d)

#define NDR_SPOOLSS_GETJOBNAMEDPROPERTYVALUE (0x6e)

#define NDR_SPOOLSS_SETJOBNAMEDPROPERTY (0x6f)

#define NDR_SPOOLSS_DELETEJOBNAMEDPROPERTY (0x70)

#define NDR_SPOOLSS_ENUMJOBNAMEDPROPERTIES (0x71)

#define NDR_SPOOLSS_72 (0x72)

#define NDR_SPOOLSS_73 (0x73)

#define NDR_SPOOLSS_LOGJOBINFOFORBRANCHOFFICE (0x74)

#define NDR_SPOOLSS_CALL_COUNT (117)
void ndr_print_spoolss_Time(struct ndr_print *ndr, const char *name, const struct spoolss_Time *r);
enum ndr_err_code ndr_push_spoolss_TimeCtr(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_TimeCtr *r);
enum ndr_err_code ndr_pull_spoolss_TimeCtr(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_TimeCtr *r);
void ndr_print_spoolss_TimeCtr(struct ndr_print *ndr, const char *name, const struct spoolss_TimeCtr *r);
void ndr_print_spoolss_ProcessorArchitecture(struct ndr_print *ndr, const char *name, enum spoolss_ProcessorArchitecture r);
void ndr_print_spoolss_ProcessorType(struct ndr_print *ndr, const char *name, enum spoolss_ProcessorType r);
void ndr_print_spoolss_MajorVersion(struct ndr_print *ndr, const char *name, enum spoolss_MajorVersion r);
void ndr_print_spoolss_MinorVersion(struct ndr_print *ndr, const char *name, enum spoolss_MinorVersion r);
enum ndr_err_code ndr_push_spoolss_PrinterStatus(struct ndr_push *ndr, ndr_flags_type ndr_flags, uint32_t r);
enum ndr_err_code ndr_pull_spoolss_PrinterStatus(struct ndr_pull *ndr, ndr_flags_type ndr_flags, uint32_t *r);
void ndr_print_spoolss_PrinterStatus(struct ndr_print *ndr, const char *name, uint32_t r);
void ndr_print_spoolss_JobStatus(struct ndr_print *ndr, const char *name, uint32_t r);
void ndr_print_spoolss_Build(struct ndr_print *ndr, const char *name, enum spoolss_Build r);
enum ndr_err_code ndr_push_spoolss_PrinterInfo0(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PrinterInfo0 *r);
enum ndr_err_code ndr_pull_spoolss_PrinterInfo0(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PrinterInfo0 *r);
void ndr_print_spoolss_PrinterInfo0(struct ndr_print *ndr, const char *name, const struct spoolss_PrinterInfo0 *r);
size_t ndr_size_spoolss_PrinterInfo0(const struct spoolss_PrinterInfo0 *r, libndr_flags flags);
void ndr_print_spoolss_DM_Signature(struct ndr_print *ndr, const char *name, enum spoolss_DM_Signature r);
void ndr_print_spoolss_DM_Layout(struct ndr_print *ndr, const char *name, enum spoolss_DM_Layout r);
void ndr_print_spoolss_DM_NupDirection(struct ndr_print *ndr, const char *name, enum spoolss_DM_NupDirection r);
void ndr_print_spoolss_DM_NupBorderFlags(struct ndr_print *ndr, const char *name, enum spoolss_DM_NupBorderFlags r);
void ndr_print_spoolss_DM_BookletFlags(struct ndr_print *ndr, const char *name, enum spoolss_DM_BookletFlags r);
void ndr_print_spoolss_DMUNI_Quality(struct ndr_print *ndr, const char *name, enum spoolss_DMUNI_Quality r);
void ndr_print_spoolss_DMUNI_Flags(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_spoolss_UNIDRVEXTRA(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_UNIDRVEXTRA *r);
enum ndr_err_code ndr_pull_spoolss_UNIDRVEXTRA(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_UNIDRVEXTRA *r);
void ndr_print_spoolss_UNIDRVEXTRA(struct ndr_print *ndr, const char *name, const struct spoolss_UNIDRVEXTRA *r);
size_t ndr_size_spoolss_UNIDRVEXTRA(const struct spoolss_UNIDRVEXTRA *r, libndr_flags flags);
void ndr_print_spoolss_DeviceModeFields(struct ndr_print *ndr, const char *name, uint32_t r);
void ndr_print_spoolss_DeviceModeSpecVersion(struct ndr_print *ndr, const char *name, enum spoolss_DeviceModeSpecVersion r);
void ndr_print_spoolss_DeviceModeOrientation(struct ndr_print *ndr, const char *name, enum spoolss_DeviceModeOrientation r);
void ndr_print_spoolss_DeviceModePaperSize(struct ndr_print *ndr, const char *name, enum spoolss_DeviceModePaperSize r);
void ndr_print_spoolss_DeviceModeDefaultSource(struct ndr_print *ndr, const char *name, enum spoolss_DeviceModeDefaultSource r);
void ndr_print_spoolss_DeviceModePrintQuality(struct ndr_print *ndr, const char *name, enum spoolss_DeviceModePrintQuality r);
void ndr_print_spoolss_DeviceModeColor(struct ndr_print *ndr, const char *name, enum spoolss_DeviceModeColor r);
void ndr_print_spoolss_DeviceModeDuplex(struct ndr_print *ndr, const char *name, enum spoolss_DeviceModeDuplex r);
void ndr_print_spoolss_DeviceModeTTOption(struct ndr_print *ndr, const char *name, enum spoolss_DeviceModeTTOption r);
void ndr_print_spoolss_DeviceModeCollate(struct ndr_print *ndr, const char *name, enum spoolss_DeviceModeCollate r);
void ndr_print_spoolss_DeviceModeNUp(struct ndr_print *ndr, const char *name, enum spoolss_DeviceModeNUp r);
void ndr_print_spoolss_DeviceModeICMMethod(struct ndr_print *ndr, const char *name, enum spoolss_DeviceModeICMMethod r);
void ndr_print_spoolss_DeviceModeICMIntent(struct ndr_print *ndr, const char *name, enum spoolss_DeviceModeICMIntent r);
void ndr_print_spoolss_DeviceModeMediaType(struct ndr_print *ndr, const char *name, enum spoolss_DeviceModeMediaType r);
void ndr_print_spoolss_DeviceModeDitherType(struct ndr_print *ndr, const char *name, enum spoolss_DeviceModeDitherType r);
enum ndr_err_code ndr_push_spoolss_DeviceMode(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_DeviceMode *r);
enum ndr_err_code ndr_pull_spoolss_DeviceMode(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_DeviceMode *r);
void ndr_print_spoolss_DeviceMode(struct ndr_print *ndr, const char *name, const struct spoolss_DeviceMode *r);
size_t ndr_size_spoolss_DeviceMode(const struct spoolss_DeviceMode *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_EnumPrinterFlags(struct ndr_push *ndr, ndr_flags_type ndr_flags, uint32_t r);
enum ndr_err_code ndr_pull_spoolss_EnumPrinterFlags(struct ndr_pull *ndr, ndr_flags_type ndr_flags, uint32_t *r);
void ndr_print_spoolss_EnumPrinterFlags(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_spoolss_PrinterAttributes(struct ndr_push *ndr, ndr_flags_type ndr_flags, uint32_t r);
enum ndr_err_code ndr_pull_spoolss_PrinterAttributes(struct ndr_pull *ndr, ndr_flags_type ndr_flags, uint32_t *r);
void ndr_print_spoolss_PrinterAttributes(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_spoolss_PrinterInfo1(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PrinterInfo1 *r);
enum ndr_err_code ndr_pull_spoolss_PrinterInfo1(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PrinterInfo1 *r);
void ndr_print_spoolss_PrinterInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_PrinterInfo1 *r);
size_t ndr_size_spoolss_PrinterInfo1(const struct spoolss_PrinterInfo1 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_PrinterInfo2(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PrinterInfo2 *r);
enum ndr_err_code ndr_pull_spoolss_PrinterInfo2(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PrinterInfo2 *r);
void ndr_print_spoolss_PrinterInfo2(struct ndr_print *ndr, const char *name, const struct spoolss_PrinterInfo2 *r);
size_t ndr_size_spoolss_PrinterInfo2(const struct spoolss_PrinterInfo2 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_PrinterInfo3(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PrinterInfo3 *r);
enum ndr_err_code ndr_pull_spoolss_PrinterInfo3(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PrinterInfo3 *r);
void ndr_print_spoolss_PrinterInfo3(struct ndr_print *ndr, const char *name, const struct spoolss_PrinterInfo3 *r);
size_t ndr_size_spoolss_PrinterInfo3(const struct spoolss_PrinterInfo3 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_PrinterInfo4(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PrinterInfo4 *r);
enum ndr_err_code ndr_pull_spoolss_PrinterInfo4(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PrinterInfo4 *r);
void ndr_print_spoolss_PrinterInfo4(struct ndr_print *ndr, const char *name, const struct spoolss_PrinterInfo4 *r);
size_t ndr_size_spoolss_PrinterInfo4(const struct spoolss_PrinterInfo4 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_PrinterInfo5(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PrinterInfo5 *r);
enum ndr_err_code ndr_pull_spoolss_PrinterInfo5(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PrinterInfo5 *r);
void ndr_print_spoolss_PrinterInfo5(struct ndr_print *ndr, const char *name, const struct spoolss_PrinterInfo5 *r);
size_t ndr_size_spoolss_PrinterInfo5(const struct spoolss_PrinterInfo5 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_PrinterInfo6(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PrinterInfo6 *r);
enum ndr_err_code ndr_pull_spoolss_PrinterInfo6(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PrinterInfo6 *r);
void ndr_print_spoolss_PrinterInfo6(struct ndr_print *ndr, const char *name, const struct spoolss_PrinterInfo6 *r);
size_t ndr_size_spoolss_PrinterInfo6(const struct spoolss_PrinterInfo6 *r, libndr_flags flags);
void ndr_print_spoolss_DsPrintAction(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_spoolss_PrinterInfo7(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PrinterInfo7 *r);
enum ndr_err_code ndr_pull_spoolss_PrinterInfo7(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PrinterInfo7 *r);
void ndr_print_spoolss_PrinterInfo7(struct ndr_print *ndr, const char *name, const struct spoolss_PrinterInfo7 *r);
size_t ndr_size_spoolss_PrinterInfo7(const struct spoolss_PrinterInfo7 *r, libndr_flags flags);
void ndr_print_spoolss_DeviceModeInfo(struct ndr_print *ndr, const char *name, const struct spoolss_DeviceModeInfo *r);
enum ndr_err_code ndr_push_spoolss_PrinterInfo(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union spoolss_PrinterInfo *r);
enum ndr_err_code ndr_pull_spoolss_PrinterInfo(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union spoolss_PrinterInfo *r);
void ndr_print_spoolss_PrinterInfo(struct ndr_print *ndr, const char *name, const union spoolss_PrinterInfo *r);
size_t ndr_size_spoolss_PrinterInfo(const union spoolss_PrinterInfo *r, uint32_t level, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_DevmodeContainer(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_DevmodeContainer *r);
enum ndr_err_code ndr_pull_spoolss_DevmodeContainer(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_DevmodeContainer *r);
void ndr_print_spoolss_DevmodeContainer(struct ndr_print *ndr, const char *name, const struct spoolss_DevmodeContainer *r);
enum ndr_err_code ndr_push_spoolss_JobInfo1(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_JobInfo1 *r);
enum ndr_err_code ndr_pull_spoolss_JobInfo1(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_JobInfo1 *r);
void ndr_print_spoolss_JobInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_JobInfo1 *r);
size_t ndr_size_spoolss_JobInfo1(const struct spoolss_JobInfo1 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_JobInfo2(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_JobInfo2 *r);
enum ndr_err_code ndr_pull_spoolss_JobInfo2(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_JobInfo2 *r);
void ndr_print_spoolss_JobInfo2(struct ndr_print *ndr, const char *name, const struct spoolss_JobInfo2 *r);
size_t ndr_size_spoolss_JobInfo2(const struct spoolss_JobInfo2 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_JobInfo3(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_JobInfo3 *r);
enum ndr_err_code ndr_pull_spoolss_JobInfo3(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_JobInfo3 *r);
void ndr_print_spoolss_JobInfo3(struct ndr_print *ndr, const char *name, const struct spoolss_JobInfo3 *r);
size_t ndr_size_spoolss_JobInfo3(const struct spoolss_JobInfo3 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_JobInfo4(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_JobInfo4 *r);
enum ndr_err_code ndr_pull_spoolss_JobInfo4(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_JobInfo4 *r);
void ndr_print_spoolss_JobInfo4(struct ndr_print *ndr, const char *name, const struct spoolss_JobInfo4 *r);
size_t ndr_size_spoolss_JobInfo4(const struct spoolss_JobInfo4 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_JobInfo(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union spoolss_JobInfo *r);
enum ndr_err_code ndr_pull_spoolss_JobInfo(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union spoolss_JobInfo *r);
void ndr_print_spoolss_JobInfo(struct ndr_print *ndr, const char *name, const union spoolss_JobInfo *r);
size_t ndr_size_spoolss_JobInfo(const union spoolss_JobInfo *r, uint32_t level, libndr_flags flags);
void ndr_print_spoolss_SetJobInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_SetJobInfo1 *r);
void ndr_print_spoolss_SetJobInfo2(struct ndr_print *ndr, const char *name, const struct spoolss_SetJobInfo2 *r);
void ndr_print_spoolss_SetJobInfo4(struct ndr_print *ndr, const char *name, const struct spoolss_SetJobInfo4 *r);
enum ndr_err_code ndr_push_spoolss_SetJobInfo(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union spoolss_SetJobInfo *r);
enum ndr_err_code ndr_pull_spoolss_SetJobInfo(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union spoolss_SetJobInfo *r);
void ndr_print_spoolss_SetJobInfo(struct ndr_print *ndr, const char *name, const union spoolss_SetJobInfo *r);
enum ndr_err_code ndr_push_spoolss_JobInfoContainer(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_JobInfoContainer *r);
enum ndr_err_code ndr_pull_spoolss_JobInfoContainer(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_JobInfoContainer *r);
void ndr_print_spoolss_JobInfoContainer(struct ndr_print *ndr, const char *name, const struct spoolss_JobInfoContainer *r);
void ndr_print_spoolss_JobControl(struct ndr_print *ndr, const char *name, enum spoolss_JobControl r);
void ndr_print_spoolss_PrinterControl(struct ndr_print *ndr, const char *name, enum spoolss_PrinterControl r);
void ndr_print_spoolss_SetPrinterInfo0(struct ndr_print *ndr, const char *name, const struct spoolss_SetPrinterInfo0 *r);
void ndr_print_spoolss_SetPrinterInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_SetPrinterInfo1 *r);
void ndr_print_spoolss_SetPrinterInfo2(struct ndr_print *ndr, const char *name, const struct spoolss_SetPrinterInfo2 *r);
void ndr_print_spoolss_SetPrinterInfo3(struct ndr_print *ndr, const char *name, const struct spoolss_SetPrinterInfo3 *r);
void ndr_print_spoolss_SetPrinterInfo4(struct ndr_print *ndr, const char *name, const struct spoolss_SetPrinterInfo4 *r);
void ndr_print_spoolss_SetPrinterInfo5(struct ndr_print *ndr, const char *name, const struct spoolss_SetPrinterInfo5 *r);
void ndr_print_spoolss_SetPrinterInfo6(struct ndr_print *ndr, const char *name, const struct spoolss_SetPrinterInfo6 *r);
void ndr_print_spoolss_SetPrinterInfo7(struct ndr_print *ndr, const char *name, const struct spoolss_SetPrinterInfo7 *r);
void ndr_print_spoolss_SetPrinterInfo8(struct ndr_print *ndr, const char *name, const struct spoolss_SetPrinterInfo8 *r);
void ndr_print_spoolss_SetPrinterInfo9(struct ndr_print *ndr, const char *name, const struct spoolss_SetPrinterInfo9 *r);
void ndr_print_spoolss_SetPrinterInfo(struct ndr_print *ndr, const char *name, const union spoolss_SetPrinterInfo *r);
enum ndr_err_code ndr_push_spoolss_SetPrinterInfoCtr(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_SetPrinterInfoCtr *r);
enum ndr_err_code ndr_pull_spoolss_SetPrinterInfoCtr(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_SetPrinterInfoCtr *r);
void ndr_print_spoolss_SetPrinterInfoCtr(struct ndr_print *ndr, const char *name, const struct spoolss_SetPrinterInfoCtr *r);
enum ndr_err_code ndr_push_spoolss_StringArray(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_StringArray *r);
enum ndr_err_code ndr_pull_spoolss_StringArray(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_StringArray *r);
void ndr_print_spoolss_StringArray(struct ndr_print *ndr, const char *name, const struct spoolss_StringArray *r);
void ndr_print_spoolss_AddDriverInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_AddDriverInfo1 *r);
enum ndr_err_code ndr_push_spoolss_DriverOSVersion(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum spoolss_DriverOSVersion r);
enum ndr_err_code ndr_pull_spoolss_DriverOSVersion(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum spoolss_DriverOSVersion *r);
void ndr_print_spoolss_DriverOSVersion(struct ndr_print *ndr, const char *name, enum spoolss_DriverOSVersion r);
void ndr_print_spoolss_AddDriverInfo2(struct ndr_print *ndr, const char *name, const struct spoolss_AddDriverInfo2 *r);
void ndr_print_spoolss_AddDriverInfo3(struct ndr_print *ndr, const char *name, const struct spoolss_AddDriverInfo3 *r);
void ndr_print_spoolss_AddDriverInfo4(struct ndr_print *ndr, const char *name, const struct spoolss_AddDriverInfo4 *r);
void ndr_print_spoolss_DriverAttributes(struct ndr_print *ndr, const char *name, uint32_t r);
void ndr_print_spoolss_AddDriverInfo6(struct ndr_print *ndr, const char *name, const struct spoolss_AddDriverInfo6 *r);
void ndr_print_spoolss_AddDriverInfo8(struct ndr_print *ndr, const char *name, const struct spoolss_AddDriverInfo8 *r);
void ndr_print_spoolss_AddDriverInfo(struct ndr_print *ndr, const char *name, const union spoolss_AddDriverInfo *r);
enum ndr_err_code ndr_push_spoolss_AddDriverInfoCtr(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_AddDriverInfoCtr *r);
enum ndr_err_code ndr_pull_spoolss_AddDriverInfoCtr(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_AddDriverInfoCtr *r);
void ndr_print_spoolss_AddDriverInfoCtr(struct ndr_print *ndr, const char *name, const struct spoolss_AddDriverInfoCtr *r);
enum ndr_err_code ndr_push_spoolss_DriverInfo1(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_DriverInfo1 *r);
enum ndr_err_code ndr_pull_spoolss_DriverInfo1(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_DriverInfo1 *r);
void ndr_print_spoolss_DriverInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_DriverInfo1 *r);
size_t ndr_size_spoolss_DriverInfo1(const struct spoolss_DriverInfo1 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_DriverInfo2(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_DriverInfo2 *r);
enum ndr_err_code ndr_pull_spoolss_DriverInfo2(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_DriverInfo2 *r);
void ndr_print_spoolss_DriverInfo2(struct ndr_print *ndr, const char *name, const struct spoolss_DriverInfo2 *r);
size_t ndr_size_spoolss_DriverInfo2(const struct spoolss_DriverInfo2 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_DriverInfo3(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_DriverInfo3 *r);
enum ndr_err_code ndr_pull_spoolss_DriverInfo3(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_DriverInfo3 *r);
void ndr_print_spoolss_DriverInfo3(struct ndr_print *ndr, const char *name, const struct spoolss_DriverInfo3 *r);
size_t ndr_size_spoolss_DriverInfo3(const struct spoolss_DriverInfo3 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_DriverInfo4(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_DriverInfo4 *r);
enum ndr_err_code ndr_pull_spoolss_DriverInfo4(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_DriverInfo4 *r);
void ndr_print_spoolss_DriverInfo4(struct ndr_print *ndr, const char *name, const struct spoolss_DriverInfo4 *r);
size_t ndr_size_spoolss_DriverInfo4(const struct spoolss_DriverInfo4 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_DriverInfo5(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_DriverInfo5 *r);
enum ndr_err_code ndr_pull_spoolss_DriverInfo5(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_DriverInfo5 *r);
void ndr_print_spoolss_DriverInfo5(struct ndr_print *ndr, const char *name, const struct spoolss_DriverInfo5 *r);
size_t ndr_size_spoolss_DriverInfo5(const struct spoolss_DriverInfo5 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_DriverInfo6(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_DriverInfo6 *r);
enum ndr_err_code ndr_pull_spoolss_DriverInfo6(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_DriverInfo6 *r);
void ndr_print_spoolss_DriverInfo6(struct ndr_print *ndr, const char *name, const struct spoolss_DriverInfo6 *r);
size_t ndr_size_spoolss_DriverInfo6(const struct spoolss_DriverInfo6 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_DriverInfo7(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_DriverInfo7 *r);
enum ndr_err_code ndr_pull_spoolss_DriverInfo7(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_DriverInfo7 *r);
void ndr_print_spoolss_DriverInfo7(struct ndr_print *ndr, const char *name, const struct spoolss_DriverInfo7 *r);
size_t ndr_size_spoolss_DriverInfo7(const struct spoolss_DriverInfo7 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_DriverInfo8(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_DriverInfo8 *r);
enum ndr_err_code ndr_pull_spoolss_DriverInfo8(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_DriverInfo8 *r);
void ndr_print_spoolss_DriverInfo8(struct ndr_print *ndr, const char *name, const struct spoolss_DriverInfo8 *r);
size_t ndr_size_spoolss_DriverInfo8(const struct spoolss_DriverInfo8 *r, libndr_flags flags);
void ndr_print_spoolss_DriverFileType(struct ndr_print *ndr, const char *name, enum spoolss_DriverFileType r);
enum ndr_err_code ndr_push_spoolss_DriverFileInfo(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_DriverFileInfo *r);
enum ndr_err_code ndr_pull_spoolss_DriverFileInfo(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_DriverFileInfo *r);
void ndr_print_spoolss_DriverFileInfo(struct ndr_print *ndr, const char *name, const struct spoolss_DriverFileInfo *r);
enum ndr_err_code ndr_push_spoolss_DriverInfo101(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_DriverInfo101 *r);
enum ndr_err_code ndr_pull_spoolss_DriverInfo101(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_DriverInfo101 *r);
void ndr_print_spoolss_DriverInfo101(struct ndr_print *ndr, const char *name, const struct spoolss_DriverInfo101 *r);
size_t ndr_size_spoolss_DriverInfo101(const struct spoolss_DriverInfo101 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_DriverInfo(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union spoolss_DriverInfo *r);
enum ndr_err_code ndr_pull_spoolss_DriverInfo(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union spoolss_DriverInfo *r);
void ndr_print_spoolss_DriverInfo(struct ndr_print *ndr, const char *name, const union spoolss_DriverInfo *r);
size_t ndr_size_spoolss_DriverInfo(const union spoolss_DriverInfo *r, uint32_t level, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_DriverDirectoryInfo1(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_DriverDirectoryInfo1 *r);
enum ndr_err_code ndr_pull_spoolss_DriverDirectoryInfo1(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_DriverDirectoryInfo1 *r);
void ndr_print_spoolss_DriverDirectoryInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_DriverDirectoryInfo1 *r);
size_t ndr_size_spoolss_DriverDirectoryInfo1(const struct spoolss_DriverDirectoryInfo1 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_DriverDirectoryInfo(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union spoolss_DriverDirectoryInfo *r);
enum ndr_err_code ndr_pull_spoolss_DriverDirectoryInfo(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union spoolss_DriverDirectoryInfo *r);
void ndr_print_spoolss_DriverDirectoryInfo(struct ndr_print *ndr, const char *name, const union spoolss_DriverDirectoryInfo *r);
size_t ndr_size_spoolss_DriverDirectoryInfo(const union spoolss_DriverDirectoryInfo *r, uint32_t level, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_PrintProcessorInfo1(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PrintProcessorInfo1 *r);
enum ndr_err_code ndr_pull_spoolss_PrintProcessorInfo1(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PrintProcessorInfo1 *r);
void ndr_print_spoolss_PrintProcessorInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_PrintProcessorInfo1 *r);
size_t ndr_size_spoolss_PrintProcessorInfo1(const struct spoolss_PrintProcessorInfo1 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_PrintProcessorInfo(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union spoolss_PrintProcessorInfo *r);
enum ndr_err_code ndr_pull_spoolss_PrintProcessorInfo(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union spoolss_PrintProcessorInfo *r);
void ndr_print_spoolss_PrintProcessorInfo(struct ndr_print *ndr, const char *name, const union spoolss_PrintProcessorInfo *r);
enum ndr_err_code ndr_push_spoolss_PrintProcessorDirectoryInfo1(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PrintProcessorDirectoryInfo1 *r);
enum ndr_err_code ndr_pull_spoolss_PrintProcessorDirectoryInfo1(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PrintProcessorDirectoryInfo1 *r);
void ndr_print_spoolss_PrintProcessorDirectoryInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_PrintProcessorDirectoryInfo1 *r);
size_t ndr_size_spoolss_PrintProcessorDirectoryInfo1(const struct spoolss_PrintProcessorDirectoryInfo1 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_PrintProcessorDirectoryInfo(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union spoolss_PrintProcessorDirectoryInfo *r);
enum ndr_err_code ndr_pull_spoolss_PrintProcessorDirectoryInfo(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union spoolss_PrintProcessorDirectoryInfo *r);
void ndr_print_spoolss_PrintProcessorDirectoryInfo(struct ndr_print *ndr, const char *name, const union spoolss_PrintProcessorDirectoryInfo *r);
size_t ndr_size_spoolss_PrintProcessorDirectoryInfo(const union spoolss_PrintProcessorDirectoryInfo *r, uint32_t level, libndr_flags flags);
void ndr_print_spoolss_DocumentInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_DocumentInfo1 *r);
void ndr_print_spoolss_DocumentInfo(struct ndr_print *ndr, const char *name, const union spoolss_DocumentInfo *r);
enum ndr_err_code ndr_push_spoolss_DocumentInfoCtr(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_DocumentInfoCtr *r);
enum ndr_err_code ndr_pull_spoolss_DocumentInfoCtr(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_DocumentInfoCtr *r);
void ndr_print_spoolss_DocumentInfoCtr(struct ndr_print *ndr, const char *name, const struct spoolss_DocumentInfoCtr *r);
enum ndr_err_code ndr_push_spoolss_OSVersion(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_OSVersion *r);
enum ndr_err_code ndr_pull_spoolss_OSVersion(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_OSVersion *r);
void ndr_print_spoolss_OSVersion(struct ndr_print *ndr, const char *name, const struct spoolss_OSVersion *r);
size_t ndr_size_spoolss_OSVersion(const struct spoolss_OSVersion *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_OSVersionEx(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_OSVersionEx *r);
enum ndr_err_code ndr_pull_spoolss_OSVersionEx(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_OSVersionEx *r);
void ndr_print_spoolss_OSVersionEx(struct ndr_print *ndr, const char *name, const struct spoolss_OSVersionEx *r);
size_t ndr_size_spoolss_OSVersionEx(const struct spoolss_OSVersionEx *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_PrinterData(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union spoolss_PrinterData *r);
enum ndr_err_code ndr_pull_spoolss_PrinterData(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union spoolss_PrinterData *r);
void ndr_print_spoolss_PrinterData(struct ndr_print *ndr, const char *name, const union spoolss_PrinterData *r);
void ndr_print_spoolss_FormFlags(struct ndr_print *ndr, const char *name, enum spoolss_FormFlags r);
void ndr_print_spoolss_FormSize(struct ndr_print *ndr, const char *name, const struct spoolss_FormSize *r);
void ndr_print_spoolss_FormArea(struct ndr_print *ndr, const char *name, const struct spoolss_FormArea *r);
enum ndr_err_code ndr_push_spoolss_FormInfo1(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_FormInfo1 *r);
enum ndr_err_code ndr_pull_spoolss_FormInfo1(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_FormInfo1 *r);
void ndr_print_spoolss_FormInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_FormInfo1 *r);
size_t ndr_size_spoolss_FormInfo1(const struct spoolss_FormInfo1 *r, libndr_flags flags);
void ndr_print_spoolss_FormStringType(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_spoolss_FormInfo2(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_FormInfo2 *r);
enum ndr_err_code ndr_pull_spoolss_FormInfo2(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_FormInfo2 *r);
void ndr_print_spoolss_FormInfo2(struct ndr_print *ndr, const char *name, const struct spoolss_FormInfo2 *r);
size_t ndr_size_spoolss_FormInfo2(const struct spoolss_FormInfo2 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_FormInfo(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union spoolss_FormInfo *r);
enum ndr_err_code ndr_pull_spoolss_FormInfo(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union spoolss_FormInfo *r);
void ndr_print_spoolss_FormInfo(struct ndr_print *ndr, const char *name, const union spoolss_FormInfo *r);
size_t ndr_size_spoolss_FormInfo(const union spoolss_FormInfo *r, uint32_t level, libndr_flags flags);
void ndr_print_spoolss_AddFormInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_AddFormInfo1 *r);
void ndr_print_spoolss_AddFormInfo2(struct ndr_print *ndr, const char *name, const struct spoolss_AddFormInfo2 *r);
void ndr_print_spoolss_AddFormInfo(struct ndr_print *ndr, const char *name, const union spoolss_AddFormInfo *r);
enum ndr_err_code ndr_push_spoolss_AddFormInfoCtr(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_AddFormInfoCtr *r);
enum ndr_err_code ndr_pull_spoolss_AddFormInfoCtr(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_AddFormInfoCtr *r);
void ndr_print_spoolss_AddFormInfoCtr(struct ndr_print *ndr, const char *name, const struct spoolss_AddFormInfoCtr *r);
enum ndr_err_code ndr_push_spoolss_PortInfo1(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PortInfo1 *r);
enum ndr_err_code ndr_pull_spoolss_PortInfo1(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PortInfo1 *r);
void ndr_print_spoolss_PortInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_PortInfo1 *r);
size_t ndr_size_spoolss_PortInfo1(const struct spoolss_PortInfo1 *r, libndr_flags flags);
void ndr_print_spoolss_PortType(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_spoolss_PortInfo2(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PortInfo2 *r);
enum ndr_err_code ndr_pull_spoolss_PortInfo2(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PortInfo2 *r);
void ndr_print_spoolss_PortInfo2(struct ndr_print *ndr, const char *name, const struct spoolss_PortInfo2 *r);
size_t ndr_size_spoolss_PortInfo2(const struct spoolss_PortInfo2 *r, libndr_flags flags);
void ndr_print_spoolss_PortStatus(struct ndr_print *ndr, const char *name, enum spoolss_PortStatus r);
void ndr_print_spoolss_PortSeverity(struct ndr_print *ndr, const char *name, enum spoolss_PortSeverity r);
enum ndr_err_code ndr_push_spoolss_PortInfo3(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PortInfo3 *r);
enum ndr_err_code ndr_pull_spoolss_PortInfo3(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PortInfo3 *r);
void ndr_print_spoolss_PortInfo3(struct ndr_print *ndr, const char *name, const struct spoolss_PortInfo3 *r);
size_t ndr_size_spoolss_PortInfo3(const struct spoolss_PortInfo3 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_PortInfoFF(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PortInfoFF *r);
enum ndr_err_code ndr_pull_spoolss_PortInfoFF(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PortInfoFF *r);
void ndr_print_spoolss_PortInfoFF(struct ndr_print *ndr, const char *name, const struct spoolss_PortInfoFF *r);
size_t ndr_size_spoolss_PortInfoFF(const struct spoolss_PortInfoFF *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_PortInfo(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union spoolss_PortInfo *r);
enum ndr_err_code ndr_pull_spoolss_PortInfo(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union spoolss_PortInfo *r);
void ndr_print_spoolss_PortInfo(struct ndr_print *ndr, const char *name, const union spoolss_PortInfo *r);
enum ndr_err_code ndr_push_spoolss_MonitorInfo1(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_MonitorInfo1 *r);
enum ndr_err_code ndr_pull_spoolss_MonitorInfo1(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_MonitorInfo1 *r);
void ndr_print_spoolss_MonitorInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_MonitorInfo1 *r);
size_t ndr_size_spoolss_MonitorInfo1(const struct spoolss_MonitorInfo1 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_MonitorInfo2(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_MonitorInfo2 *r);
enum ndr_err_code ndr_pull_spoolss_MonitorInfo2(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_MonitorInfo2 *r);
void ndr_print_spoolss_MonitorInfo2(struct ndr_print *ndr, const char *name, const struct spoolss_MonitorInfo2 *r);
size_t ndr_size_spoolss_MonitorInfo2(const struct spoolss_MonitorInfo2 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_MonitorInfo(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union spoolss_MonitorInfo *r);
enum ndr_err_code ndr_pull_spoolss_MonitorInfo(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union spoolss_MonitorInfo *r);
void ndr_print_spoolss_MonitorInfo(struct ndr_print *ndr, const char *name, const union spoolss_MonitorInfo *r);
enum ndr_err_code ndr_push_spoolss_SetPortInfo1(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_SetPortInfo1 *r);
enum ndr_err_code ndr_pull_spoolss_SetPortInfo1(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_SetPortInfo1 *r);
void ndr_print_spoolss_SetPortInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_SetPortInfo1 *r);
size_t ndr_size_spoolss_SetPortInfo1(const struct spoolss_SetPortInfo1 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_SetPortInfo2(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_SetPortInfo2 *r);
enum ndr_err_code ndr_pull_spoolss_SetPortInfo2(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_SetPortInfo2 *r);
void ndr_print_spoolss_SetPortInfo2(struct ndr_print *ndr, const char *name, const struct spoolss_SetPortInfo2 *r);
size_t ndr_size_spoolss_SetPortInfo2(const struct spoolss_SetPortInfo2 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_SetPortInfo3(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_SetPortInfo3 *r);
enum ndr_err_code ndr_pull_spoolss_SetPortInfo3(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_SetPortInfo3 *r);
void ndr_print_spoolss_SetPortInfo3(struct ndr_print *ndr, const char *name, const struct spoolss_SetPortInfo3 *r);
size_t ndr_size_spoolss_SetPortInfo3(const struct spoolss_SetPortInfo3 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_SetPortInfoFF(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_SetPortInfoFF *r);
enum ndr_err_code ndr_pull_spoolss_SetPortInfoFF(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_SetPortInfoFF *r);
void ndr_print_spoolss_SetPortInfoFF(struct ndr_print *ndr, const char *name, const struct spoolss_SetPortInfoFF *r);
size_t ndr_size_spoolss_SetPortInfoFF(const struct spoolss_SetPortInfoFF *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_SetPortInfoContainer(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_SetPortInfoContainer *r);
enum ndr_err_code ndr_pull_spoolss_SetPortInfoContainer(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_SetPortInfoContainer *r);
void ndr_print_spoolss_SetPortInfoContainer(struct ndr_print *ndr, const char *name, const struct spoolss_SetPortInfoContainer *r);
void ndr_print_UNIVERSAL_FONT_ID(struct ndr_print *ndr, const char *name, const struct UNIVERSAL_FONT_ID *r);
enum ndr_err_code ndr_push_UNIVERSAL_FONT_ID_ctr(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct UNIVERSAL_FONT_ID_ctr *r);
enum ndr_err_code ndr_pull_UNIVERSAL_FONT_ID_ctr(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct UNIVERSAL_FONT_ID_ctr *r);
void ndr_print_UNIVERSAL_FONT_ID_ctr(struct ndr_print *ndr, const char *name, const struct UNIVERSAL_FONT_ID_ctr *r);
enum ndr_err_code ndr_push_spoolss_MonitorContainer(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_MonitorContainer *r);
enum ndr_err_code ndr_pull_spoolss_MonitorContainer(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_MonitorContainer *r);
void ndr_print_spoolss_MonitorContainer(struct ndr_print *ndr, const char *name, const struct spoolss_MonitorContainer *r);
enum ndr_err_code ndr_push_spoolss_PrintProcDataTypesInfo1(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PrintProcDataTypesInfo1 *r);
enum ndr_err_code ndr_pull_spoolss_PrintProcDataTypesInfo1(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PrintProcDataTypesInfo1 *r);
void ndr_print_spoolss_PrintProcDataTypesInfo1(struct ndr_print *ndr, const char *name, const struct spoolss_PrintProcDataTypesInfo1 *r);
size_t ndr_size_spoolss_PrintProcDataTypesInfo1(const struct spoolss_PrintProcDataTypesInfo1 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_PrintProcDataTypesInfo(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union spoolss_PrintProcDataTypesInfo *r);
enum ndr_err_code ndr_pull_spoolss_PrintProcDataTypesInfo(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union spoolss_PrintProcDataTypesInfo *r);
void ndr_print_spoolss_PrintProcDataTypesInfo(struct ndr_print *ndr, const char *name, const union spoolss_PrintProcDataTypesInfo *r);
void ndr_print_spoolss_PrinterChangeFlags(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_spoolss_PortVarContainer(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PortVarContainer *r);
enum ndr_err_code ndr_pull_spoolss_PortVarContainer(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PortVarContainer *r);
void ndr_print_spoolss_PortVarContainer(struct ndr_print *ndr, const char *name, const struct spoolss_PortVarContainer *r);
enum ndr_err_code ndr_push_spoolss_JobNotifyField(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum spoolss_JobNotifyField r);
enum ndr_err_code ndr_pull_spoolss_JobNotifyField(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum spoolss_JobNotifyField *r);
void ndr_print_spoolss_JobNotifyField(struct ndr_print *ndr, const char *name, enum spoolss_JobNotifyField r);
enum ndr_err_code ndr_push_spoolss_PrintNotifyField(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum spoolss_PrintNotifyField r);
enum ndr_err_code ndr_pull_spoolss_PrintNotifyField(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum spoolss_PrintNotifyField *r);
void ndr_print_spoolss_PrintNotifyField(struct ndr_print *ndr, const char *name, enum spoolss_PrintNotifyField r);
void ndr_print_spoolss_NotifyType(struct ndr_print *ndr, const char *name, enum spoolss_NotifyType r);
void ndr_print_spoolss_Field(struct ndr_print *ndr, const char *name, const union spoolss_Field *r);
void ndr_print_spoolss_NotifyOptionType(struct ndr_print *ndr, const char *name, const struct spoolss_NotifyOptionType *r);
void ndr_print_spoolssNotifyOptionFlags(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_spoolss_NotifyOption(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_NotifyOption *r);
enum ndr_err_code ndr_pull_spoolss_NotifyOption(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_NotifyOption *r);
void ndr_print_spoolss_NotifyOption(struct ndr_print *ndr, const char *name, const struct spoolss_NotifyOption *r);
void ndr_print_spoolss_NotifyString(struct ndr_print *ndr, const char *name, const struct spoolss_NotifyString *r);
void ndr_print_spoolss_NotifyTable(struct ndr_print *ndr, const char *name, enum spoolss_NotifyTable r);
void ndr_print_spoolss_NotifyData(struct ndr_print *ndr, const char *name, const union spoolss_NotifyData *r);
void ndr_print_spoolss_Notify(struct ndr_print *ndr, const char *name, const struct spoolss_Notify *r);
enum ndr_err_code ndr_push_spoolss_NotifyInfo(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_NotifyInfo *r);
enum ndr_err_code ndr_pull_spoolss_NotifyInfo(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_NotifyInfo *r);
void ndr_print_spoolss_NotifyInfo(struct ndr_print *ndr, const char *name, const struct spoolss_NotifyInfo *r);
void ndr_print_spoolss_ReplyPrinterInfo(struct ndr_print *ndr, const char *name, const union spoolss_ReplyPrinterInfo *r);
void ndr_print_spoolss_PrinterNotifyFlags(struct ndr_print *ndr, const char *name, uint32_t r);
void ndr_print_spoolss_UserLevel1(struct ndr_print *ndr, const char *name, const struct spoolss_UserLevel1 *r);
void ndr_print_spoolss_UserLevel2(struct ndr_print *ndr, const char *name, const struct spoolss_UserLevel2 *r);
void ndr_print_spoolss_UserLevel3(struct ndr_print *ndr, const char *name, const struct spoolss_UserLevel3 *r);
void ndr_print_spoolss_UserLevel(struct ndr_print *ndr, const char *name, const union spoolss_UserLevel *r);
enum ndr_err_code ndr_push_spoolss_UserLevelCtr(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_UserLevelCtr *r);
enum ndr_err_code ndr_pull_spoolss_UserLevelCtr(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_UserLevelCtr *r);
void ndr_print_spoolss_UserLevelCtr(struct ndr_print *ndr, const char *name, const struct spoolss_UserLevelCtr *r);
void ndr_print_spoolss_AccessRights(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_spoolss_PrinterEnumValues(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PrinterEnumValues *r);
enum ndr_err_code ndr_pull_spoolss_PrinterEnumValues(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PrinterEnumValues *r);
void ndr_print_spoolss_PrinterEnumValues(struct ndr_print *ndr, const char *name, const struct spoolss_PrinterEnumValues *r);
size_t ndr_size_spoolss_PrinterEnumValues(const struct spoolss_PrinterEnumValues *r, libndr_flags flags);
void ndr_print_spoolss_KeyNames(struct ndr_print *ndr, const char *name, const union spoolss_KeyNames *r);
enum ndr_err_code ndr_push_spoolss_DeleteDriverFlags(struct ndr_push *ndr, ndr_flags_type ndr_flags, uint32_t r);
enum ndr_err_code ndr_pull_spoolss_DeleteDriverFlags(struct ndr_pull *ndr, ndr_flags_type ndr_flags, uint32_t *r);
void ndr_print_spoolss_DeleteDriverFlags(struct ndr_print *ndr, const char *name, uint32_t r);
void ndr_print_spoolss_PortProtocol(struct ndr_print *ndr, const char *name, enum spoolss_PortProtocol r);
enum ndr_err_code ndr_push_spoolss_PortData1(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PortData1 *r);
enum ndr_err_code ndr_pull_spoolss_PortData1(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PortData1 *r);
void ndr_print_spoolss_PortData1(struct ndr_print *ndr, const char *name, const struct spoolss_PortData1 *r);
size_t ndr_size_spoolss_PortData1(const struct spoolss_PortData1 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_PortData2(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PortData2 *r);
enum ndr_err_code ndr_pull_spoolss_PortData2(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PortData2 *r);
void ndr_print_spoolss_PortData2(struct ndr_print *ndr, const char *name, const struct spoolss_PortData2 *r);
size_t ndr_size_spoolss_PortData2(const struct spoolss_PortData2 *r, libndr_flags flags);
enum ndr_err_code ndr_push_spoolss_MonitorUi(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_MonitorUi *r);
enum ndr_err_code ndr_pull_spoolss_MonitorUi(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_MonitorUi *r);
void ndr_print_spoolss_MonitorUi(struct ndr_print *ndr, const char *name, const struct spoolss_MonitorUi *r);
void ndr_print_spoolss_AddPrinterDriverExFlags(struct ndr_print *ndr, const char *name, uint32_t r);
void ndr_print_RPC_BINARY_CONTAINER(struct ndr_print *ndr, const char *name, const struct RPC_BINARY_CONTAINER *r);
void ndr_print_RPC_BIDI_DATA_u(struct ndr_print *ndr, const char *name, const union RPC_BIDI_DATA_u *r);
void ndr_print_RPC_BIDI_DATA(struct ndr_print *ndr, const char *name, const struct RPC_BIDI_DATA *r);
void ndr_print_RPC_BIDI_REQUEST_DATA(struct ndr_print *ndr, const char *name, const struct RPC_BIDI_REQUEST_DATA *r);
enum ndr_err_code ndr_push_RPC_BIDI_REQUEST_CONTAINER(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct RPC_BIDI_REQUEST_CONTAINER *r);
enum ndr_err_code ndr_pull_RPC_BIDI_REQUEST_CONTAINER(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct RPC_BIDI_REQUEST_CONTAINER *r);
void ndr_print_RPC_BIDI_REQUEST_CONTAINER(struct ndr_print *ndr, const char *name, const struct RPC_BIDI_REQUEST_CONTAINER *r);
void ndr_print_RPC_BIDI_RESPONSE_DATA(struct ndr_print *ndr, const char *name, const struct RPC_BIDI_RESPONSE_DATA *r);
enum ndr_err_code ndr_push_RPC_BIDI_RESPONSE_CONTAINER(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct RPC_BIDI_RESPONSE_CONTAINER *r);
enum ndr_err_code ndr_pull_RPC_BIDI_RESPONSE_CONTAINER(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct RPC_BIDI_RESPONSE_CONTAINER *r);
void ndr_print_RPC_BIDI_RESPONSE_CONTAINER(struct ndr_print *ndr, const char *name, const struct RPC_BIDI_RESPONSE_CONTAINER *r);
enum ndr_err_code ndr_push_spoolss_CorePrinterDriver(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_CorePrinterDriver *r);
enum ndr_err_code ndr_pull_spoolss_CorePrinterDriver(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_CorePrinterDriver *r);
void ndr_print_spoolss_CorePrinterDriver(struct ndr_print *ndr, const char *name, const struct spoolss_CorePrinterDriver *r);
void ndr_print_spoolss_EPrintPropertyType(struct ndr_print *ndr, const char *name, enum spoolss_EPrintPropertyType r);
void ndr_print_propertyBlob(struct ndr_print *ndr, const char *name, const struct propertyBlob *r);
void ndr_print_spoolss_PrintPropertyValueUnion(struct ndr_print *ndr, const char *name, const union spoolss_PrintPropertyValueUnion *r);
enum ndr_err_code ndr_push_spoolss_PrintPropertyValue(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PrintPropertyValue *r);
enum ndr_err_code ndr_pull_spoolss_PrintPropertyValue(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PrintPropertyValue *r);
void ndr_print_spoolss_PrintPropertyValue(struct ndr_print *ndr, const char *name, const struct spoolss_PrintPropertyValue *r);
enum ndr_err_code ndr_push_spoolss_PrintNamedProperty(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_PrintNamedProperty *r);
enum ndr_err_code ndr_pull_spoolss_PrintNamedProperty(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_PrintNamedProperty *r);
void ndr_print_spoolss_PrintNamedProperty(struct ndr_print *ndr, const char *name, const struct spoolss_PrintNamedProperty *r);
void ndr_print_EBranchOfficeJobEventType(struct ndr_print *ndr, const char *name, enum EBranchOfficeJobEventType r);
void ndr_print_spoolss_BranchOfficeJobDataPrinted(struct ndr_print *ndr, const char *name, const struct spoolss_BranchOfficeJobDataPrinted *r);
void ndr_print_spoolss_BranchOfficeJobDataRendered(struct ndr_print *ndr, const char *name, const struct spoolss_BranchOfficeJobDataRendered *r);
void ndr_print_spoolss_BranchOfficeJobDataError(struct ndr_print *ndr, const char *name, const struct spoolss_BranchOfficeJobDataError *r);
void ndr_print_spoolss_BranchOfficeJobDataPipelineFailed(struct ndr_print *ndr, const char *name, const struct spoolss_BranchOfficeJobDataPipelineFailed *r);
void ndr_print_spoolss_BranchOfficeLogOfflineFileFull(struct ndr_print *ndr, const char *name, const struct spoolss_BranchOfficeLogOfflineFileFull *r);
void ndr_print_spoolss_BranchOfficeJobInfo(struct ndr_print *ndr, const char *name, const union spoolss_BranchOfficeJobInfo *r);
void ndr_print_spoolss_BranchOfficeJobData(struct ndr_print *ndr, const char *name, const struct spoolss_BranchOfficeJobData *r);
enum ndr_err_code ndr_push_spoolss_BranchOfficeJobDataContainer(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct spoolss_BranchOfficeJobDataContainer *r);
enum ndr_err_code ndr_pull_spoolss_BranchOfficeJobDataContainer(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct spoolss_BranchOfficeJobDataContainer *r);
void ndr_print_spoolss_BranchOfficeJobDataContainer(struct ndr_print *ndr, const char *name, const struct spoolss_BranchOfficeJobDataContainer *r);
enum ndr_err_code ndr_push__spoolss_EnumPrinters(struct ndr_push *ndr, ndr_flags_type flags, const struct _spoolss_EnumPrinters *r);
enum ndr_err_code ndr_pull__spoolss_EnumPrinters(struct ndr_pull *ndr, ndr_flags_type flags, struct _spoolss_EnumPrinters *r);
void ndr_print__spoolss_EnumPrinters(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct _spoolss_EnumPrinters *r);
enum ndr_err_code ndr_push___spoolss_EnumPrinters(struct ndr_push *ndr, ndr_flags_type flags, const struct __spoolss_EnumPrinters *r);
enum ndr_err_code ndr_pull___spoolss_EnumPrinters(struct ndr_pull *ndr, ndr_flags_type flags, struct __spoolss_EnumPrinters *r);
void ndr_print___spoolss_EnumPrinters(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct __spoolss_EnumPrinters *r);
enum ndr_err_code ndr_push_spoolss_EnumPrinters(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_EnumPrinters *r);
enum ndr_err_code ndr_pull_spoolss_EnumPrinters(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_EnumPrinters *r);
void ndr_print_spoolss_EnumPrinters(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_EnumPrinters *r);
enum ndr_err_code ndr_push_spoolss_OpenPrinter(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_OpenPrinter *r);
enum ndr_err_code ndr_pull_spoolss_OpenPrinter(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_OpenPrinter *r);
void ndr_print_spoolss_OpenPrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_OpenPrinter *r);
void ndr_print_spoolss_SetJob(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_SetJob *r);
void ndr_print_spoolss_GetJob(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_GetJob *r);
enum ndr_err_code ndr_push__spoolss_EnumJobs(struct ndr_push *ndr, ndr_flags_type flags, const struct _spoolss_EnumJobs *r);
enum ndr_err_code ndr_pull__spoolss_EnumJobs(struct ndr_pull *ndr, ndr_flags_type flags, struct _spoolss_EnumJobs *r);
void ndr_print__spoolss_EnumJobs(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct _spoolss_EnumJobs *r);
enum ndr_err_code ndr_push___spoolss_EnumJobs(struct ndr_push *ndr, ndr_flags_type flags, const struct __spoolss_EnumJobs *r);
enum ndr_err_code ndr_pull___spoolss_EnumJobs(struct ndr_pull *ndr, ndr_flags_type flags, struct __spoolss_EnumJobs *r);
void ndr_print___spoolss_EnumJobs(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct __spoolss_EnumJobs *r);
enum ndr_err_code ndr_push_spoolss_EnumJobs(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_EnumJobs *r);
enum ndr_err_code ndr_pull_spoolss_EnumJobs(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_EnumJobs *r);
void ndr_print_spoolss_EnumJobs(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_EnumJobs *r);
void ndr_print_spoolss_AddPrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_AddPrinter *r);
void ndr_print_spoolss_DeletePrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_DeletePrinter *r);
enum ndr_err_code ndr_push_spoolss_SetPrinter(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_SetPrinter *r);
enum ndr_err_code ndr_pull_spoolss_SetPrinter(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_SetPrinter *r);
void ndr_print_spoolss_SetPrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_SetPrinter *r);
enum ndr_err_code ndr_push_spoolss_GetPrinter(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_GetPrinter *r);
enum ndr_err_code ndr_pull_spoolss_GetPrinter(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_GetPrinter *r);
void ndr_print_spoolss_GetPrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_GetPrinter *r);
void ndr_print_spoolss_AddPrinterDriver(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_AddPrinterDriver *r);
enum ndr_err_code ndr_push__spoolss_EnumPrinterDrivers(struct ndr_push *ndr, ndr_flags_type flags, const struct _spoolss_EnumPrinterDrivers *r);
enum ndr_err_code ndr_pull__spoolss_EnumPrinterDrivers(struct ndr_pull *ndr, ndr_flags_type flags, struct _spoolss_EnumPrinterDrivers *r);
void ndr_print__spoolss_EnumPrinterDrivers(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct _spoolss_EnumPrinterDrivers *r);
enum ndr_err_code ndr_push___spoolss_EnumPrinterDrivers(struct ndr_push *ndr, ndr_flags_type flags, const struct __spoolss_EnumPrinterDrivers *r);
enum ndr_err_code ndr_pull___spoolss_EnumPrinterDrivers(struct ndr_pull *ndr, ndr_flags_type flags, struct __spoolss_EnumPrinterDrivers *r);
void ndr_print___spoolss_EnumPrinterDrivers(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct __spoolss_EnumPrinterDrivers *r);
enum ndr_err_code ndr_push_spoolss_EnumPrinterDrivers(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_EnumPrinterDrivers *r);
enum ndr_err_code ndr_pull_spoolss_EnumPrinterDrivers(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_EnumPrinterDrivers *r);
void ndr_print_spoolss_EnumPrinterDrivers(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_EnumPrinterDrivers *r);
void ndr_print_spoolss_GetPrinterDriver(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_GetPrinterDriver *r);
enum ndr_err_code ndr_push_spoolss_GetPrinterDriverDirectory(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_GetPrinterDriverDirectory *r);
enum ndr_err_code ndr_pull_spoolss_GetPrinterDriverDirectory(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_GetPrinterDriverDirectory *r);
void ndr_print_spoolss_GetPrinterDriverDirectory(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_GetPrinterDriverDirectory *r);
void ndr_print_spoolss_DeletePrinterDriver(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_DeletePrinterDriver *r);
void ndr_print_spoolss_AddPrintProcessor(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_AddPrintProcessor *r);
enum ndr_err_code ndr_push__spoolss_EnumPrintProcessors(struct ndr_push *ndr, ndr_flags_type flags, const struct _spoolss_EnumPrintProcessors *r);
enum ndr_err_code ndr_pull__spoolss_EnumPrintProcessors(struct ndr_pull *ndr, ndr_flags_type flags, struct _spoolss_EnumPrintProcessors *r);
void ndr_print__spoolss_EnumPrintProcessors(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct _spoolss_EnumPrintProcessors *r);
enum ndr_err_code ndr_push___spoolss_EnumPrintProcessors(struct ndr_push *ndr, ndr_flags_type flags, const struct __spoolss_EnumPrintProcessors *r);
enum ndr_err_code ndr_pull___spoolss_EnumPrintProcessors(struct ndr_pull *ndr, ndr_flags_type flags, struct __spoolss_EnumPrintProcessors *r);
void ndr_print___spoolss_EnumPrintProcessors(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct __spoolss_EnumPrintProcessors *r);
enum ndr_err_code ndr_push_spoolss_EnumPrintProcessors(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_EnumPrintProcessors *r);
enum ndr_err_code ndr_pull_spoolss_EnumPrintProcessors(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_EnumPrintProcessors *r);
void ndr_print_spoolss_EnumPrintProcessors(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_EnumPrintProcessors *r);
void ndr_print_spoolss_GetPrintProcessorDirectory(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_GetPrintProcessorDirectory *r);
void ndr_print_spoolss_StartDocPrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_StartDocPrinter *r);
void ndr_print_spoolss_StartPagePrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_StartPagePrinter *r);
void ndr_print_spoolss_WritePrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_WritePrinter *r);
void ndr_print_spoolss_EndPagePrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_EndPagePrinter *r);
void ndr_print_spoolss_AbortPrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_AbortPrinter *r);
void ndr_print_spoolss_ReadPrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_ReadPrinter *r);
void ndr_print_spoolss_EndDocPrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_EndDocPrinter *r);
void ndr_print_spoolss_AddJob(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_AddJob *r);
void ndr_print_spoolss_ScheduleJob(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_ScheduleJob *r);
enum ndr_err_code ndr_push_spoolss_GetPrinterData(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_GetPrinterData *r);
enum ndr_err_code ndr_pull_spoolss_GetPrinterData(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_GetPrinterData *r);
void ndr_print_spoolss_GetPrinterData(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_GetPrinterData *r);
void ndr_print_spoolss_SetPrinterData(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_SetPrinterData *r);
void ndr_print_spoolss_WaitForPrinterChange(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_WaitForPrinterChange *r);
enum ndr_err_code ndr_push_spoolss_ClosePrinter(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_ClosePrinter *r);
enum ndr_err_code ndr_pull_spoolss_ClosePrinter(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_ClosePrinter *r);
void ndr_print_spoolss_ClosePrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_ClosePrinter *r);
void ndr_print_spoolss_AddForm(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_AddForm *r);
void ndr_print_spoolss_DeleteForm(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_DeleteForm *r);
void ndr_print_spoolss_GetForm(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_GetForm *r);
void ndr_print_spoolss_SetForm(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_SetForm *r);
enum ndr_err_code ndr_push__spoolss_EnumForms(struct ndr_push *ndr, ndr_flags_type flags, const struct _spoolss_EnumForms *r);
enum ndr_err_code ndr_pull__spoolss_EnumForms(struct ndr_pull *ndr, ndr_flags_type flags, struct _spoolss_EnumForms *r);
void ndr_print__spoolss_EnumForms(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct _spoolss_EnumForms *r);
enum ndr_err_code ndr_push___spoolss_EnumForms(struct ndr_push *ndr, ndr_flags_type flags, const struct __spoolss_EnumForms *r);
enum ndr_err_code ndr_pull___spoolss_EnumForms(struct ndr_pull *ndr, ndr_flags_type flags, struct __spoolss_EnumForms *r);
void ndr_print___spoolss_EnumForms(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct __spoolss_EnumForms *r);
enum ndr_err_code ndr_push_spoolss_EnumForms(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_EnumForms *r);
enum ndr_err_code ndr_pull_spoolss_EnumForms(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_EnumForms *r);
void ndr_print_spoolss_EnumForms(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_EnumForms *r);
enum ndr_err_code ndr_push__spoolss_EnumPorts(struct ndr_push *ndr, ndr_flags_type flags, const struct _spoolss_EnumPorts *r);
enum ndr_err_code ndr_pull__spoolss_EnumPorts(struct ndr_pull *ndr, ndr_flags_type flags, struct _spoolss_EnumPorts *r);
void ndr_print__spoolss_EnumPorts(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct _spoolss_EnumPorts *r);
enum ndr_err_code ndr_push___spoolss_EnumPorts(struct ndr_push *ndr, ndr_flags_type flags, const struct __spoolss_EnumPorts *r);
enum ndr_err_code ndr_pull___spoolss_EnumPorts(struct ndr_pull *ndr, ndr_flags_type flags, struct __spoolss_EnumPorts *r);
void ndr_print___spoolss_EnumPorts(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct __spoolss_EnumPorts *r);
enum ndr_err_code ndr_push_spoolss_EnumPorts(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_EnumPorts *r);
enum ndr_err_code ndr_pull_spoolss_EnumPorts(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_EnumPorts *r);
void ndr_print_spoolss_EnumPorts(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_EnumPorts *r);
enum ndr_err_code ndr_push__spoolss_EnumMonitors(struct ndr_push *ndr, ndr_flags_type flags, const struct _spoolss_EnumMonitors *r);
enum ndr_err_code ndr_pull__spoolss_EnumMonitors(struct ndr_pull *ndr, ndr_flags_type flags, struct _spoolss_EnumMonitors *r);
void ndr_print__spoolss_EnumMonitors(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct _spoolss_EnumMonitors *r);
enum ndr_err_code ndr_push___spoolss_EnumMonitors(struct ndr_push *ndr, ndr_flags_type flags, const struct __spoolss_EnumMonitors *r);
enum ndr_err_code ndr_pull___spoolss_EnumMonitors(struct ndr_pull *ndr, ndr_flags_type flags, struct __spoolss_EnumMonitors *r);
void ndr_print___spoolss_EnumMonitors(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct __spoolss_EnumMonitors *r);
enum ndr_err_code ndr_push_spoolss_EnumMonitors(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_EnumMonitors *r);
enum ndr_err_code ndr_pull_spoolss_EnumMonitors(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_EnumMonitors *r);
void ndr_print_spoolss_EnumMonitors(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_EnumMonitors *r);
void ndr_print_spoolss_AddPort(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_AddPort *r);
void ndr_print_spoolss_ConfigurePort(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_ConfigurePort *r);
void ndr_print_spoolss_DeletePort(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_DeletePort *r);
void ndr_print_spoolss_CreatePrinterIC(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_CreatePrinterIC *r);
void ndr_print_spoolss_PlayGDIScriptOnPrinterIC(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_PlayGDIScriptOnPrinterIC *r);
void ndr_print_spoolss_DeletePrinterIC(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_DeletePrinterIC *r);
void ndr_print_spoolss_AddPrinterConnection(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_AddPrinterConnection *r);
void ndr_print_spoolss_DeletePrinterConnection(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_DeletePrinterConnection *r);
void ndr_print_spoolss_PrinterMessageBox(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_PrinterMessageBox *r);
void ndr_print_spoolss_AddMonitor(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_AddMonitor *r);
void ndr_print_spoolss_DeleteMonitor(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_DeleteMonitor *r);
void ndr_print_spoolss_DeletePrintProcessor(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_DeletePrintProcessor *r);
void ndr_print_spoolss_AddPrintProvidor(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_AddPrintProvidor *r);
void ndr_print_spoolss_DeletePrintProvidor(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_DeletePrintProvidor *r);
enum ndr_err_code ndr_push__spoolss_EnumPrintProcessorDataTypes(struct ndr_push *ndr, ndr_flags_type flags, const struct _spoolss_EnumPrintProcessorDataTypes *r);
enum ndr_err_code ndr_pull__spoolss_EnumPrintProcessorDataTypes(struct ndr_pull *ndr, ndr_flags_type flags, struct _spoolss_EnumPrintProcessorDataTypes *r);
void ndr_print__spoolss_EnumPrintProcessorDataTypes(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct _spoolss_EnumPrintProcessorDataTypes *r);
enum ndr_err_code ndr_push___spoolss_EnumPrintProcessorDataTypes(struct ndr_push *ndr, ndr_flags_type flags, const struct __spoolss_EnumPrintProcessorDataTypes *r);
enum ndr_err_code ndr_pull___spoolss_EnumPrintProcessorDataTypes(struct ndr_pull *ndr, ndr_flags_type flags, struct __spoolss_EnumPrintProcessorDataTypes *r);
void ndr_print___spoolss_EnumPrintProcessorDataTypes(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct __spoolss_EnumPrintProcessorDataTypes *r);
enum ndr_err_code ndr_push_spoolss_EnumPrintProcessorDataTypes(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_EnumPrintProcessorDataTypes *r);
enum ndr_err_code ndr_pull_spoolss_EnumPrintProcessorDataTypes(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_EnumPrintProcessorDataTypes *r);
void ndr_print_spoolss_EnumPrintProcessorDataTypes(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_EnumPrintProcessorDataTypes *r);
void ndr_print_spoolss_ResetPrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_ResetPrinter *r);
enum ndr_err_code ndr_push_spoolss_GetPrinterDriver2(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_GetPrinterDriver2 *r);
enum ndr_err_code ndr_pull_spoolss_GetPrinterDriver2(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_GetPrinterDriver2 *r);
void ndr_print_spoolss_GetPrinterDriver2(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_GetPrinterDriver2 *r);
void ndr_print_spoolss_FindFirstPrinterChangeNotification(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_FindFirstPrinterChangeNotification *r);
void ndr_print_spoolss_FindNextPrinterChangeNotification(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_FindNextPrinterChangeNotification *r);
enum ndr_err_code ndr_push_spoolss_FindClosePrinterNotify(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_FindClosePrinterNotify *r);
enum ndr_err_code ndr_pull_spoolss_FindClosePrinterNotify(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_FindClosePrinterNotify *r);
void ndr_print_spoolss_FindClosePrinterNotify(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_FindClosePrinterNotify *r);
void ndr_print_spoolss_RouterFindFirstPrinterChangeNotificationOld(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_RouterFindFirstPrinterChangeNotificationOld *r);
enum ndr_err_code ndr_push_spoolss_ReplyOpenPrinter(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_ReplyOpenPrinter *r);
enum ndr_err_code ndr_pull_spoolss_ReplyOpenPrinter(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_ReplyOpenPrinter *r);
void ndr_print_spoolss_ReplyOpenPrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_ReplyOpenPrinter *r);
void ndr_print_spoolss_RouterReplyPrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_RouterReplyPrinter *r);
enum ndr_err_code ndr_push_spoolss_ReplyClosePrinter(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_ReplyClosePrinter *r);
enum ndr_err_code ndr_pull_spoolss_ReplyClosePrinter(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_ReplyClosePrinter *r);
void ndr_print_spoolss_ReplyClosePrinter(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_ReplyClosePrinter *r);
void ndr_print_spoolss_AddPortEx(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_AddPortEx *r);
void ndr_print_spoolss_RouterFindFirstPrinterChangeNotification(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_RouterFindFirstPrinterChangeNotification *r);
void ndr_print_spoolss_SpoolerInit(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_SpoolerInit *r);
void ndr_print_spoolss_ResetPrinterEx(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_ResetPrinterEx *r);
enum ndr_err_code ndr_push_spoolss_RemoteFindFirstPrinterChangeNotifyEx(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_RemoteFindFirstPrinterChangeNotifyEx *r);
enum ndr_err_code ndr_pull_spoolss_RemoteFindFirstPrinterChangeNotifyEx(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_RemoteFindFirstPrinterChangeNotifyEx *r);
void ndr_print_spoolss_RemoteFindFirstPrinterChangeNotifyEx(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_RemoteFindFirstPrinterChangeNotifyEx *r);
void ndr_print_spoolss_RouterReplyPrinterEx(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_RouterReplyPrinterEx *r);
enum ndr_err_code ndr_push_spoolss_RouterRefreshPrinterChangeNotify(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_RouterRefreshPrinterChangeNotify *r);
enum ndr_err_code ndr_pull_spoolss_RouterRefreshPrinterChangeNotify(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_RouterRefreshPrinterChangeNotify *r);
void ndr_print_spoolss_RouterRefreshPrinterChangeNotify(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_RouterRefreshPrinterChangeNotify *r);
void ndr_print_spoolss_44(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_44 *r);
enum ndr_err_code ndr_push_spoolss_OpenPrinterEx(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_OpenPrinterEx *r);
enum ndr_err_code ndr_pull_spoolss_OpenPrinterEx(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_OpenPrinterEx *r);
void ndr_print_spoolss_OpenPrinterEx(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_OpenPrinterEx *r);
void ndr_print_spoolss_AddPrinterEx(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_AddPrinterEx *r);
void ndr_print_spoolss_SetPort(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_SetPort *r);
void ndr_print_spoolss_EnumPrinterData(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_EnumPrinterData *r);
void ndr_print_spoolss_DeletePrinterData(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_DeletePrinterData *r);
void ndr_print_spoolss_4a(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_4a *r);
void ndr_print_spoolss_4b(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_4b *r);
void ndr_print_spoolss_4c(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_4c *r);
void ndr_print_spoolss_SetPrinterDataEx(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_SetPrinterDataEx *r);
void ndr_print_spoolss_GetPrinterDataEx(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_GetPrinterDataEx *r);
enum ndr_err_code ndr_push__spoolss_EnumPrinterDataEx(struct ndr_push *ndr, ndr_flags_type flags, const struct _spoolss_EnumPrinterDataEx *r);
enum ndr_err_code ndr_pull__spoolss_EnumPrinterDataEx(struct ndr_pull *ndr, ndr_flags_type flags, struct _spoolss_EnumPrinterDataEx *r);
void ndr_print__spoolss_EnumPrinterDataEx(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct _spoolss_EnumPrinterDataEx *r);
enum ndr_err_code ndr_push___spoolss_EnumPrinterDataEx(struct ndr_push *ndr, ndr_flags_type flags, const struct __spoolss_EnumPrinterDataEx *r);
enum ndr_err_code ndr_pull___spoolss_EnumPrinterDataEx(struct ndr_pull *ndr, ndr_flags_type flags, struct __spoolss_EnumPrinterDataEx *r);
void ndr_print___spoolss_EnumPrinterDataEx(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct __spoolss_EnumPrinterDataEx *r);
enum ndr_err_code ndr_push_spoolss_EnumPrinterDataEx(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_EnumPrinterDataEx *r);
enum ndr_err_code ndr_pull_spoolss_EnumPrinterDataEx(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_EnumPrinterDataEx *r);
void ndr_print_spoolss_EnumPrinterDataEx(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_EnumPrinterDataEx *r);
enum ndr_err_code ndr_push_spoolss_EnumPrinterKey(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_EnumPrinterKey *r);
enum ndr_err_code ndr_pull_spoolss_EnumPrinterKey(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_EnumPrinterKey *r);
void ndr_print_spoolss_EnumPrinterKey(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_EnumPrinterKey *r);
void ndr_print_spoolss_DeletePrinterDataEx(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_DeletePrinterDataEx *r);
void ndr_print_spoolss_DeletePrinterKey(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_DeletePrinterKey *r);
void ndr_print_spoolss_53(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_53 *r);
void ndr_print_spoolss_DeletePrinterDriverEx(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_DeletePrinterDriverEx *r);
void ndr_print_spoolss_AddPerMachineConnection(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_AddPerMachineConnection *r);
void ndr_print_spoolss_DeletePerMachineConnection(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_DeletePerMachineConnection *r);
enum ndr_err_code ndr_push__spoolss_EnumPerMachineConnections(struct ndr_push *ndr, ndr_flags_type flags, const struct _spoolss_EnumPerMachineConnections *r);
enum ndr_err_code ndr_pull__spoolss_EnumPerMachineConnections(struct ndr_pull *ndr, ndr_flags_type flags, struct _spoolss_EnumPerMachineConnections *r);
void ndr_print__spoolss_EnumPerMachineConnections(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct _spoolss_EnumPerMachineConnections *r);
enum ndr_err_code ndr_push___spoolss_EnumPerMachineConnections(struct ndr_push *ndr, ndr_flags_type flags, const struct __spoolss_EnumPerMachineConnections *r);
enum ndr_err_code ndr_pull___spoolss_EnumPerMachineConnections(struct ndr_pull *ndr, ndr_flags_type flags, struct __spoolss_EnumPerMachineConnections *r);
void ndr_print___spoolss_EnumPerMachineConnections(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct __spoolss_EnumPerMachineConnections *r);
enum ndr_err_code ndr_push_spoolss_EnumPerMachineConnections(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_EnumPerMachineConnections *r);
enum ndr_err_code ndr_pull_spoolss_EnumPerMachineConnections(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_EnumPerMachineConnections *r);
void ndr_print_spoolss_EnumPerMachineConnections(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_EnumPerMachineConnections *r);
void ndr_print_spoolss_XcvData(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_XcvData *r);
enum ndr_err_code ndr_push_spoolss_AddPrinterDriverEx(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_AddPrinterDriverEx *r);
enum ndr_err_code ndr_pull_spoolss_AddPrinterDriverEx(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_AddPrinterDriverEx *r);
void ndr_print_spoolss_AddPrinterDriverEx(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_AddPrinterDriverEx *r);
void ndr_print_spoolss_5a(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_5a *r);
void ndr_print_spoolss_5b(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_5b *r);
void ndr_print_spoolss_5c(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_5c *r);
void ndr_print_spoolss_5d(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_5d *r);
void ndr_print_spoolss_5e(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_5e *r);
void ndr_print_spoolss_5f(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_5f *r);
void ndr_print_spoolss_60(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_60 *r);
void ndr_print_spoolss_SendRecvBidiData(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_SendRecvBidiData *r);
void ndr_print_spoolss_62(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_62 *r);
void ndr_print_spoolss_63(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_63 *r);
void ndr_print_spoolss_64(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_64 *r);
void ndr_print_spoolss_65(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_65 *r);
enum ndr_err_code ndr_push_spoolss_GetCorePrinterDrivers(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_GetCorePrinterDrivers *r);
enum ndr_err_code ndr_pull_spoolss_GetCorePrinterDrivers(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_GetCorePrinterDrivers *r);
void ndr_print_spoolss_GetCorePrinterDrivers(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_GetCorePrinterDrivers *r);
void ndr_print_spoolss_67(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_67 *r);
void ndr_print_spoolss_GetPrinterDriverPackagePath(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_GetPrinterDriverPackagePath *r);
void ndr_print_spoolss_69(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_69 *r);
void ndr_print_spoolss_6a(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_6a *r);
void ndr_print_spoolss_6b(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_6b *r);
void ndr_print_spoolss_6c(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_6c *r);
void ndr_print_spoolss_6d(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_6d *r);
void ndr_print_spoolss_GetJobNamedPropertyValue(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_GetJobNamedPropertyValue *r);
enum ndr_err_code ndr_push_spoolss_SetJobNamedProperty(struct ndr_push *ndr, ndr_flags_type flags, const struct spoolss_SetJobNamedProperty *r);
enum ndr_err_code ndr_pull_spoolss_SetJobNamedProperty(struct ndr_pull *ndr, ndr_flags_type flags, struct spoolss_SetJobNamedProperty *r);
void ndr_print_spoolss_SetJobNamedProperty(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_SetJobNamedProperty *r);
void ndr_print_spoolss_DeleteJobNamedProperty(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_DeleteJobNamedProperty *r);
void ndr_print_spoolss_EnumJobNamedProperties(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_EnumJobNamedProperties *r);
void ndr_print_spoolss_72(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_72 *r);
void ndr_print_spoolss_73(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_73 *r);
void ndr_print_spoolss_LogJobInfoForBranchOffice(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct spoolss_LogJobInfoForBranchOffice *r);
#endif /* _HEADER_NDR_spoolss */
