#ifndef __SOURCE4_LDAP_SERVER_PROTO_H__
#define __SOURCE4_LDAP_SERVER_PROTO_H__

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
/* This file was automatically generated by mkproto.pl. DO NOT EDIT */

/* this file contains prototypes for functions that are private 
 * to this subsystem or library. These functions should not be 
 * used outside this particular subsystem! */


/* The following definitions come from ../../source4/ldap_server/ldap_server.c  */

void ldapsrv_recv(struct stream_connection *c, uint16_t flags);
void ldapsrv_notification_retry_setup(struct ldapsrv_service *service, bool force);
NTSTATUS server_service_ldap_init(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/ldap_server/ldap_backend.c  */

int ldapsrv_backend_Init(struct ldapsrv_connection *conn,
			      char **errstring);
struct ldapsrv_reply *ldapsrv_init_reply(struct ldapsrv_call *call, uint8_t type);
NTSTATUS ldapsrv_queue_reply(struct ldapsrv_call *call, struct ldapsrv_reply *reply);
NTSTATUS ldapsrv_do_call(struct ldapsrv_call *call);

/* The following definitions come from ../../source4/ldap_server/ldap_bind.c  */

NTSTATUS ldapsrv_BindRequest(struct ldapsrv_call *call);
NTSTATUS ldapsrv_UnbindRequest(struct ldapsrv_call *call);

/* The following definitions come from ../../source4/ldap_server/ldap_extended.c  */

NTSTATUS ldapsrv_ExtendedRequest(struct ldapsrv_call *call);
#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2)

#endif /* __SOURCE4_LDAP_SERVER_PROTO_H__ */

