/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/source3/librpc/gen_ndr/rpc_host.h"

#ifndef _HEADER_NDR_rpc_host_msg
#define _HEADER_NDR_rpc_host_msg

extern const struct ndr_interface_table ndr_table_rpc_host_msg;
#define NDR_RPC_HOST_MSG_CALL_COUNT (0)
enum ndr_err_code ndr_push_rpc_host_client(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct rpc_host_client *r);
enum ndr_err_code ndr_pull_rpc_host_client(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct rpc_host_client *r);
void ndr_print_rpc_host_client(struct ndr_print *ndr, const char *name, const struct rpc_host_client *r);
enum ndr_err_code ndr_push_rpc_worker_status(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct rpc_worker_status *r);
enum ndr_err_code ndr_pull_rpc_worker_status(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct rpc_worker_status *r);
void ndr_print_rpc_worker_status(struct ndr_print *ndr, const char *name, const struct rpc_worker_status *r);
#endif /* _HEADER_NDR_rpc_host_msg */
